#!/bin/sh
# Note:
#	SenaoWrt FW version is 2.X.X
#	SmartOpenWrt FW version is equal or greater than 3.X.X

ver_prefix=$(cat /etc/version | grep ^2 | cut -d'.' -f1)
[ -f /etc/config/wifi ] && iwconfig 2>/dev/null |grep Failsafe 1>/dev/null && {
	# Failsafe dosen't support config migration
	ver_prefix="0"
}

# Only do config migration from SenaoWrt => SmartOpenWrt
[ "${ver_prefix}" = "2" ] && {
	config_migration="config_migrate.sh"
	config_list="system wifi_schedule network snmpd systime sysProductInfo ntpclient luci lighttpd fastroaming emailalert dropbear dhcp6c dhcp apcontroller cfg_cli wireless firewall portal"
	support_model="enh1750ext ews210ap ews300ap ews310ap ews320ap ews350ap ews360ap ews500ap ews510ap ews650ap ews660ap ews860ap"
	support=0

	# Check supported model
	for model in $support_model
	do
		match=$(cat /etc/modelname |grep -w "$model")
		[ "${match}" = "${model}" ] && {
			echo "Checked: $model, support config migration!" > /dev/console
			support=1
			break
		}
	done

	# do config migration
	[ "$support" = "1" ] && {
		echo "Start config migration from SenaoWrt => SmartOpenWrt ...." > /dev/console

		# remove old files
		rm -rf /tmp/convert/
		rm /tmp/_sys/sysupgrade.tgz

		[ -f "${config_migration}" ] && {
			echo "Convert start...." > /dev/console
			chmod a+x "${config_migration}" && sh "${config_migration}"
		}

		ls -al /tmp/convert/ && {
			echo "rm orignal /etc/config/*" > /dev/console
			rm -rf /etc/config/*

			echo "cp /tmp/convert/* to /etc/config/" > /dev/console
			for i in $config_list
			do
				cp -f /tmp/convert/$i /etc/config/$i
			done
		} || {
			echo "Convert fail...." > /dev/console
		}

		backup_files="/etc/config/ /etc/lighttpd/lighttpd.conf /etc/dropbear/ /etc/opkg.conf /etc/webpasswd /etc/passwd /etc/shadow"
		echo "Backup files list: ${backup_files}" > /dev/console

		tar czf /tmp/_sys/sysupgrade.tgz ${backup_files} && {
			echo "Backup configuration to /tmp/_sys/sysupgrade.tgz done...." > /dev/console
		} || {
			echo "Backup configuration to /tmp/_sys/sysupgrade.tgz fail...." > /dev/console
		}

	} || {
		echo "This Senaowrt model is not supported by config migration, do nothing...." > /dev/console
	}

} || {

	[ "${ver_prefix}" = "0" ] && {
		# Failsafe
		echo "This Failsafe mode, config migration by pass...." > /dev/console
	} || {
		# SmartOpenWrt => SmartOpenWrt
		echo "This is SmartOpenWrt FW, config migration by pass...." > /dev/console
	}

}


rootfs_mtd="$(cat /proc/mtd | grep \"rootfs\" | cut -d : -f 1)"
rootfs_size="$(($( ( fw_printenv | grep ^rootfs_size= | cut -d = -f 2 ) 2>&- )))"
sysfolder="/tmp/_sys"
custom="$(cat /proc/mtd | grep "custom" | cut -d ":" -f 1 | sed -e "s/^mtd/mtdblock/g")"
if [ ! -f "/rom/note" -a -n "${rootfs_mtd}" -a ${rootfs_size} -gt 0 ]; then
	. /lib/functions/boot.sh && 
	umount -l /jffs && 
	pivot /rom /mnt && 
	umount -l /mnt && 
	{
		dd if=/dev/${rootfs_mtd} of=/tmp/root.squashfs bs=${rootfs_size} count=1 && 
		mount /tmp/root.squashfs /mnt && 
		pivot /mnt /rom && 
		umount -l /rom
	} 2>&- || true && 
	ramoverlay && 
	[ -n "${flag_disable_umount}" ] && touch ${flag_disable_umount}
fi


if [ ! -e $sysfolder/sku ]; then
	if [ -e /etc/sku ]; then
		local tmp="$(cat /etc/sku)"
	else
		local tmp="INT"
	fi
	printf $tmp > $sysfolder/sku 
fi

