#!/bin/sh
model_name=$(cat /etc/modelname)
upper_modelname=$(echo $model_name | awk '{print toupper($0)}')
sp_str="'\\\\''"		# for special char (') convert to four char ('\'')

# ./config_migrate.sh 1    for debug
[ -n "$1" ] && console_debug_wifi_only=1 || console_debug_wifi_only=0

[ $console_debug_wifi_only == "0" ] && {
	portal_config="/tmp/convert/portal"
	system_config="/tmp/convert/system"
	firewall_config="/tmp/convert/firewall"
	wifischedule_section_name="/tmp/convert/wifi_schedule"
	network_section_name="/tmp/convert/network"
	snmpd_section_name="/tmp/convert/snmpd"
	systime_section_name="/tmp/convert/systime"
	ntpclient_section_name="/tmp/convert/ntpclient"
	luci_section_name="/tmp/convert/luci"
	lighttpd_section_name="/tmp/convert/lighttpd"
	fastroaming_section_name="/tmp/convert/fastroaming"
	emailalert_section_name="/tmp/convert/emailalert"
	dropbear_section_name="/tmp/convert/dropbear"
	dhcp6c_section_name="/tmp/convert/dhcp6c"
	dhcp_section_name="/tmp/convert/dhcp"
	apcontroller_section_name="/tmp/convert/apcontroller"
	cli_section_name="/tmp/convert/cfg_cli"
	wifi_section_name="/tmp/convert/wireless"
	rewrite_passwd="/etc/passwd"
	etc_shadow="/etc/shadow"
} || {
	# console debug for wifi configuration only.
	wifi_section_name="/dev/console"

	output="/dev/null"

	portal_config=$output
	system_config=$output
	firewall_config=$output
	wifischedule_section_name=$output
	network_section_name=$output
	snmpd_section_name=$output
	systime_section_name=$output
	ntpclient_section_name=$output
	luci_section_name=$output
	lighttpd_section_name=$output
	fastroaming_section_name=$output
	emailalert_section_name=$output
	dropbear_section_name=$output
	dhcp6c_section_name=$output
	dhcp_section_name=$output
	apcontroller_section_name=$output
	cli_section_name=$output
	rewrite_passwd=$output
	etc_shadow=$output
}



# Format: "zonenumber/timezone/zonename"
timezone_list="\
0/UTC12/Kwajalein \
1/UTC11/Midway_Island,_Samoa \
2/UTC10/Hawaii \
3/NAST9/Alaska \
4/PST8/Pacific_Time \
5/UTC7/Arizona \
6/MST7/Mountain_Time \
7/UTC6/Mexico \
8/CST6/Central_Time \
9/UTC5/Colombia,_Panama \
10/EST5/Eastern_Time \
11/UTC4/Bolivia,_Venezuela \
12/AST4/Atlantic_Time \
13/BRWST4/Brazil_West \
14/NST3:30/Newfoundland \
15/WGST3/Greenland \
16/BRST3/Brazil_East \
17/UTC3/Guyana \
18/UTC2/Mid-Atlantic \
19/STD1/Azores \
20/UTC0/Gambia,_Liberia,_Morocco \
21/GMT0/England \
22/UTC-1/Tunisia \
23/STD-1/France,_Germany,_Italy \
24/UTC-2/South_Africa \
25/STD-2/Greece,_Ukraine,_Romania,_Turkey \
26/UTC-3/Iraq,_Jordan,_Kuwait \
27/EEST-3/Finland \
28/UTC-4/Armenia \
29/UTC-5/Pakistan,_Russia \
30/UTC-5:30/Bombay,_Calcutta,_Madras,_New_Delhi \
31/UTC-6/Bangladesh,_Russia \
32/UTC-7/Thailand,_Russia \
33/UTC-8/China,_Hong_Kong,_Western_Australia,_Singapore,_Taiwan,_Russia \
34/UTC-9/Japan,_Korea \
35/CST-9:30/South_Australia \
36/UTC-10/Guam,_Russia \
37/STD-10/Australia \
38/UTC-11/Solomon_Islands \
39/UTC-12/Fiji \
40/NZST-12/New_Zealand"

# Format: "senao/smart"
rate_list="\
6M/0x0b0b0b0b \
9M/0x0f0f0f0f \
12M/0x0a0a0a0a \
18M/0x0e0e0e0e \
24M/0x09090909 \
36M/0x0d0d0d0d \
48M/0x08080808 \
54M/0x0c0c0c0c \
MCS0/0x80808080 \
MCS1/0x81818181 \
MCS2/0x82828282 \
MCS3/0x83838383 \
MCS4/0x84848484 \
MCS5/0x85858585 \
MCS6/0x86868686 \
MCS7/0x8787878 \
MCS8/0x88888888 \
MCS9/0x89898989 \
MCS10/0x8a8a8a8a \
MCS11/0x8b8b8b8b \
MCS12/0x8c8c8c8c \
MCS13/0x8d8d8d8d \
MCS14/0x8e8e8e8e \
MCS15/0x8f8f8f8f \
MCS16/0x90909090 \
MCS17/0x91919191 \
MCS18/0x92929292 \
MCS19/0x93939393 \
MCS20/0x94949494 \
MCS21/0x95959595 \
MCS22/0x96969696 \
MCS23/0x97979797 \
auto/0x0"

search_zone() {
	for i in $timezone_list
	do
		[ $(echo $i |cut -d"/" -f2) == "$1" ] && {
			export zonenumber=$(echo $i |cut -d"/" -f1)
			export zonename=$(echo $i |cut -d"/" -f3| sed 's/_/ /g')
			break
		}
	done
}

search_rate() {
	for i in $rate_list
	do
		if [  $(echo $i |cut -d"/" -f1) == "$1" ]; then
			export tmp_rate=$(echo $i |cut -d"/" -f2)
			break
		fi
	done
}

model_features="ews210ap:2T2R:enable_accounting ews300ap:2T2R:enable_accounting ews310ap:2T2R:enable_accounting ews320ap:3T3R:enable_accounting ews350ap:2T2R:enable_accounting ews360ap:3T3R:enable_accounting ews510ap:2T2R:enable_accounting ews500ap:2T2R:enable_accounting ews650ap:2T2R:enable_accounting ews660ap:3T3R:enable_accounting ews860ap:3T3R:enable_accounting"
for models in $model_features
do
	is_support_radius_account=0
	dut_model_name="$(echo $models |  awk -F":" '{ print $1 }')"
	dut_ant_num="$(echo $models |  awk -F":" '{ print $2 }')"
	dut_ac_en="$(echo $models |  awk -F":" '{ print $3 }')"

	case $dut_ant_num in
		"1T1R")
			nss_value=1
		;;
		"2T2R")
			nss_value=2
		;;
		"3T3R")
			nss_value=3
		;;
	esac

	if [ "$dut_model_name" == "$model_name" ]; then
		if [ "$dut_ac_en" == "enable_accounting" ]; then
			is_support_radius_account=1
		fi
		break
	fi
done



# main start
mkdir -p /tmp/convert



# /etc/config/portal
echo "Converting portal config ........" > /dev/console
echo "config portal 'general'" >> $portal_config
echo "	option radiusServer '$(uci get wireless.wifi0.PortalRadiusServer)'" >> $portal_config
echo "	option radiusServer2 '0.0.0.0'" >> $portal_config
echo "	option radiusPort '$(uci get wireless.wifi0.PortalRadiusPort)'" >> $portal_config
echo "	option accountingServer '$(uci get wireless.wifi0.PortalAccountingServer)'" >> $portal_config
echo "	option accountingEnable '$(uci get wireless.wifi0.PortalAccountingEnable)'" >> $portal_config
echo "	option accountingPort '$(uci get wireless.wifi0.PortalAccountingPort)'" >> $portal_config
echo "	option idleTimeout '$(uci get wireless.wifi0.PortalIdleTimeout)'" >> $portal_config
echo "	option idleTimeoutEnable '$(uci get wireless.wifi0.PortalIdleTimeoutEnable)'" >> $portal_config
echo "	option uamformat '$(uci get wireless.wifi0.PortalUamformat)'" >> $portal_config
echo "	option localAuth '$(uci get wireless.wifi0.PortalLocalAuth)'" >> $portal_config
echo "	option port '$(uci get wireless.wifi0.PortalPort)'" >> $portal_config
echo "	option httpsEnable '$(uci get wireless.wifi0.PortalHttpsEnable)'" >> $portal_config
echo "	option accountingInterval '$(uci get wireless.wifi0.PortalAccountingInterval)'" >> $portal_config
echo "	option enable '$(uci get wireless.wifi0.PortalEnable)'" >> $portal_config
echo "	option userurl '$(uci get wireless.wifi0.PortalUserurl)'" >> $portal_config
echo "	option walledGardenPages '$(uci get wireless.wifi0.PortalWalledGardenPages)'" >> $portal_config
echo "	option radiusSecret '$(uci get wireless.wifi0.PortalRadiusSecret)'" >> $portal_config
echo "	option externalServer '$(uci get wireless.wifi0.PortalExternalServer)'" >> $portal_config
echo "	option externalSecret '$(uci get wireless.wifi0.PortalExternalSecret)'" >> $portal_config
echo "	option accountingSecret '$(uci get wireless.wifi0.PortalAccountingSecret)'" >> $portal_config
echo "	option radiusSecret2 ''" >> $portal_config
echo "	option sessionTimeout '$(uci get wireless.wifi0.PortalSessionTimeout)'" >> $portal_config
echo "	option sessionTimeoutEnable '$(uci get wireless.wifi0.PortalSessionTimeoutEnable)'" >> $portal_config
echo "	option loginType '$(uci get wireless.wifi0.PortalLoginType)'" >> $portal_config
echo "	option walledGardenEnable '$(uci get wireless.wifi0.PortalWalledGardenEnable)'" >> $portal_config
var="$(uci get wireless.wifi0.PortalAuthType)"
echo "	option authType '${var:-0}'" >> $portal_config




# /etc/config/system
echo "Converting system config ........" > /dev/console
echo "config system" > $system_config
echo "	option hostname '$(uci get system.system.hostname)'" >> $system_config
timezone="$(uci get system.system.timezone)"
search_zone $timezone
echo "	option timezone '$timezone'" >> $system_config
echo "	option SystemName '$(uci get system.system.SystemName)'" >> $system_config
loglevel=$(uci get system.log.LogLevel)
echo "	option LogLevel '`expr ${loglevel} + 1`'" >> $system_config
echo "	option syslog_enable '$(uci get system.log.LogLocalEnable)'" >> $system_config
echo "	option remotelog_enable '$(uci get system.log.LogServerEnable)'" >> $system_config
echo "	option zonename '$zonename'" >> $system_config
echo "	option zonenumber '$zonenumber'" >> $system_config
echo "	option log_ip '$(uci get system.system.log_ip)'" >> $system_config
echo "" >> $system_config
echo "config timeserver ntp" >> $system_config
echo "	option server '$(uci show ntpclient|grep hostname|cut -d"=" -f2)'" >> $system_config
echo "	option enable_server '$(uci show ntpclient|grep TimeSettingMode|cut -d"=" -f2)'" >> $system_config
echo "" >> $system_config
echo "config telnet 'telnet'" >> $system_config
echo "	option enable 'on'" >> $system_config
echo "" >> $system_config
echo "config led 'power_led'" >> $system_config
echo "	option name 'Power'" >> $system_config
echo "	option sysfs 'power_led'" >> $system_config
pled="$(uci get led.led.PowerLED)"
[ $pled == "0" ] && pled="1" || pled="0"
echo "	option default '$pled'" >> $system_config
echo "	option trigger 'none'" >> $system_config
echo "	option brightness '1'" >> $system_config
echo "" >> $system_config
echo "config led 'lan1_led'" >> $system_config
echo "	option name 'LAN1'" >> $system_config
echo "	option sysfs 'lan1_led'" >> $system_config
lanled="$(uci get led.led.LANLED)"
[ $lanled == "0" ] && lanled="1" || lanled="0"
echo "	option default '$lanled'" >> $system_config
echo "" >> $system_config
echo "config led 'wifi0_led'" >> $system_config
echo "	option name 'WIFI0'" >> $system_config
echo "	option sysfs 'wlan_2g_led'" >> $system_config
wlanled0="$(uci get led.led.WLAN0LED)"
[ $wlanled0 == "0" ] && wlanled0="1" || wlanled0="0"
echo "	option default '$wlanled0'" >> $system_config
echo "	option trigger 'none'" >> $system_config
echo "	option brightness '1'" >> $system_config
echo "" >> $system_config
echo "config led 'wifi1_led'" >> $system_config
echo "	option name 'WIFI1'" >> $system_config
echo "	option sysfs 'wlan_5g_led'" >> $system_config
wlanled1="$(uci get led.led.WLAN1LED)"
[ $wlanled1 == "0" ] && wlanled1="1" || wlanled1="0"
echo "	option default '$wlanled1'" >> $system_config
echo "	option trigger 'none'" >> $system_config
echo "	option brightness '1'" >> $system_config
# MESH LED
echo "config led 'mesh_led'" >> $system_config
echo "  option name 'MESH'" >> $system_config
echo "  option sysfs 'mode_led'" >> $system_config
echo "  option default '0'" >> $system_config
echo "" >> $system_config
echo "config button 'reset_pressed'" >> $system_config
echo "	option button 'reset'" >> $system_config
echo "	option action 'pressed'" >> $system_config
echo "	option time '0'" >> $system_config
echo "	option handler '/usr/sbin/reset_led.sh 10 &'" >> $system_config
echo "" >> $system_config
echo "config button 'reset_released'" >> $system_config
echo "	option button 'reset'" >> $system_config
echo "	option action 'released'" >> $system_config
echo "	option least '10'" >> $system_config
echo "	option handler 'rm -rf /overlay/* && echo reset to default >> /dev/console && sleep 3 && reboot'" >> $system_config
echo "" >> $system_config
echo "config button 'reboot'" >> $system_config
echo "	option button 'reset'" >> $system_config
echo "	option action 'released'" >> $system_config
echo "	option min '1'" >> $system_config
echo "	option max '5'" >> $system_config
echo "	option handler 'reboot && echo reboot system >> /dev/console'" >> $system_config
echo "" >> $system_config
echo "option 'factory' 'factory'" >> $system_config
echo "	option init '1'" >> $system_config
echo "" >> $system_config



# /etc/config/firewall
: :||:<<\COMMENTS
echo "Converting firewall config ........" > /dev/console
echo "config defaults" > $firewall_config
echo "	option syn_flood	0" >> $firewall_config
echo "	option input		ACCEPT" >> $firewall_config
echo "	option output		ACCEPT" >> $firewall_config
echo "	option forward		DROP" >> $firewall_config
echo "# Uncomment this line to disable ipv6 rules" >> $firewall_config
echo "#	option disable_ipv6	1" >> $firewall_config
echo "" >> $firewall_config
echo "config zone" >> $firewall_config
echo "	option name		lan" >> $firewall_config
echo "	list network		'lan'" >> $firewall_config
echo "	option input		ACCEPT" >> $firewall_config
echo "	option output		ACCEPT" >> $firewall_config
echo "	option forward		DROP" >> $firewall_config
echo "	option masq		1" >> $firewall_config
echo "	option mtu_fix		1" >> $firewall_config
echo "" >> $firewall_config
echo "# We need to accept udp packets on port 68," >> $firewall_config
echo "# see https://dev.openwrt.org/ticket/4108" >> $firewall_config
echo "config rule" >> $firewall_config
echo "	option name		Allow-DHCP-Renew" >> $firewall_config
echo "	option src		lan" >> $firewall_config
echo "	option proto		udp" >> $firewall_config
echo "	option dest_port	68" >> $firewall_config
echo "	option target		ACCEPT" >> $firewall_config
echo "	option family		ipv4" >> $firewall_config
echo "" >> $firewall_config
echo "# Allow IPv4 ping" >> $firewall_config
echo "config rule" >> $firewall_config
echo "	option name		Allow-Ping" >> $firewall_config
echo "	option src		lan" >> $firewall_config
echo "	option proto		icmp" >> $firewall_config
echo "	option icmp_type	echo-request" >> $firewall_config
echo "	option family		ipv4" >> $firewall_config
echo "	option target		ACCEPT" >> $firewall_config
echo "" >> $firewall_config
echo "# include a file with users custom iptables rules" >> $firewall_config
echo "config include" >> $firewall_config
echo "	option path /etc/firewall.user" >> $firewall_config
echo "" >> $firewall_config
echo "# Allow DHCPv6 replies" >> $firewall_config
echo "# see https://dev.openwrt.org/ticket/10381" >> $firewall_config
echo "config rule" >> $firewall_config
echo "	option name		Allow-DHCPv6" >> $firewall_config
echo "	option src		lan" >> $firewall_config
echo "	option proto		udp" >> $firewall_config
echo "	option src_ip		fe80::/10" >> $firewall_config
echo "	option src_port		547" >> $firewall_config
echo "	option dest_ip		fe80::/10" >> $firewall_config
echo "	option dest_port	546" >> $firewall_config
echo "	option family		ipv6" >> $firewall_config
echo "	option target		ACCEPT" >> $firewall_config
echo "" >> $firewall_config
echo "# Allow essential incoming IPv6 ICMP traffic" >> $firewall_config
echo "config rule" >> $firewall_config
echo "	option name		Allow-ICMPv6-Input" >> $firewall_config
echo "	option src		lan" >> $firewall_config
echo "	option proto	icmp" >> $firewall_config
echo "	list icmp_type		echo-request" >> $firewall_config
echo "	list icmp_type		echo-reply" >> $firewall_config
echo "	list icmp_type		destination-unreachable" >> $firewall_config
echo "	list icmp_type		packet-too-big" >> $firewall_config
echo "	list icmp_type		time-exceeded" >> $firewall_config
echo "	list icmp_type		bad-header" >> $firewall_config
echo "	list icmp_type		unknown-header-type" >> $firewall_config
echo "	list icmp_type		router-solicitation" >> $firewall_config
echo "	list icmp_type		neighbour-solicitation" >> $firewall_config
echo "	list icmp_type		router-advertisement" >> $firewall_config
echo "	list icmp_type		neighbour-advertisement" >> $firewall_config
echo "	option family		ipv6" >> $firewall_config
echo "	option target		ACCEPT" >> $firewall_config
echo "" >> $firewall_config
echo "# Allow essential forwarded IPv6 ICMP traffic" >> $firewall_config
echo "config rule" >> $firewall_config
echo "	option name		Allow-ICMPv6-Forward" >> $firewall_config
echo "	option src		lan" >> $firewall_config
echo "	option dest		*" >> $firewall_config
echo "	option proto		icmp" >> $firewall_config
echo "	list icmp_type		echo-request" >> $firewall_config
echo "	list icmp_type		echo-reply" >> $firewall_config
echo "	list icmp_type		destination-unreachable" >> $firewall_config
echo "	list icmp_type		packet-too-big" >> $firewall_config
echo "	list icmp_type		time-exceeded" >> $firewall_config
echo "	list icmp_type		bad-header" >> $firewall_config
echo "	list icmp_type		unknown-header-type" >> $firewall_config
echo "	option family		ipv6" >> $firewall_config
echo "	option target		ACCEPT" >> $firewall_config
echo "" >> $firewall_config
echo "config zone" >> $firewall_config
echo "	option name 'guest'" >> $firewall_config
echo "	list network 'guest'" >> $firewall_config
echo "	option input 'DROP'" >> $firewall_config
echo "	option output 'ACCEPT'" >> $firewall_config
echo "	option forward 'DROP'" >> $firewall_config
echo "" >> $firewall_config
echo "config forwarding" >> $firewall_config
echo "	option src 'guest'" >> $firewall_config
echo "	option dest 'lan'" >> $firewall_config
echo "" >> $firewall_config
echo "config rule" >> $firewall_config
echo "	option name 'Allow DNS Queries'" >> $firewall_config
echo "	option src 'guest'" >> $firewall_config
echo "	option dest_port '53'" >> $firewall_config
echo "	option proto 'tcpudp'" >> $firewall_config
echo "	option target 'ACCEPT'" >> $firewall_config
echo "" >> $firewall_config
echo "config rule" >> $firewall_config
echo "	option name 'Allow DHCP request'" >> $firewall_config
echo "	option src 'guest'" >> $firewall_config
echo "	option src_port '67-68'" >> $firewall_config
echo "	option dest_port '67-68'" >> $firewall_config
echo "	option proto 'udp'" >> $firewall_config
echo "	option target 'ACCEPT'" >> $firewall_config
echo "" >> $firewall_config
COMMENTS


#/etc/config/wifi_schedule
echo "Converting wifi_schedule config ........" > /dev/console

#section reboot
wifi_schedule_reboot_table="name day time rebootTime status"
echo "config 'reboottbl' 'reboot'" > $wifischedule_section_name
for uci_token_wifi_schedule in $wifi_schedule_reboot_table; do
	temp_value="$(uci get wirelessAvailability.reboot.$uci_token_wifi_schedule)"
	[ -n "$temp_value" ] && echo "	option $uci_token_wifi_schedule '$temp_value'" >> $wifischedule_section_name ||
	{
		case $uci_token_wifi_schedule in
		"name")
			echo "	option $uci_token_wifi_schedule 'reboot'" >> $wifischedule_section_name
		;;
		"day")
			echo "	option $uci_token_wifi_schedule ''" >> $wifischedule_section_name
		;;
		"time")
			echo "	option $uci_token_wifi_schedule '0:0'" >> $wifischedule_section_name
		;;
		"rebootTime")
			echo "	option $uci_token_wifi_schedule ''" >> $wifischedule_section_name
		;;
		"status")
			echo "	option $uci_token_wifi_schedule '0'" >> $wifischedule_section_name
		;;
		esac
	}
done
echo "" >> $wifischedule_section_name

#section wireless
transfer_ifname()
{
	ifname=$1
	len=${#ifname}
	if [ $len -eq 4 ]; then
		#echo "2.4G" > /dev/ttyS0
		var=$(echo $temp_value | cut -c 4)
 		if [ $var -ne 0 ]; then
			var="0"$var
		fi
	else
		#echo "5G" > /dev/ttyS0
		var=$(echo $temp_value | cut -c 5)
		if [ $var -eq 0 ]; then
			var=1
		else
			var="1"$var
		fi
	fi
	echo $var
}

wifi_schedule_wireless_table="ScheduleEnable iface radio templates init rules"
tmp_files="/tmp/handle_uci_wifi_schedule"
echo "config 'schedule' 'wireless'" >> $wifischedule_section_name
for uci_token_wifi_schedule in $wifi_schedule_wireless_table; do
	temp_value="$(uci get wirelessAvailability.wireless.$uci_token_wifi_schedule)"
	[ -n "$temp_value" ] && {
		if [ "$uci_token_wifi_schedule" == "iface" ]; then
			transfer_ifname $temp_value
			echo "	option $uci_token_wifi_schedule 'ath$var'" >> $wifischedule_section_name
		else
			echo "	option $uci_token_wifi_schedule '$temp_value'" >> $wifischedule_section_name
		fi 
	} || {
		case $uci_token_wifi_schedule in
			"ScheduleEnable")
				echo "	option $uci_token_wifi_schedule '0'" >> $wifischedule_section_name
			;;
			"iface")
				echo "	option $uci_token_wifi_schedule ''" >> $wifischedule_section_name
			;;
			"radio")
				echo "	option $uci_token_wifi_schedule '0'" >> $wifischedule_section_name
			;;
			"templates")
				echo "	option $uci_token_wifi_schedule '0'" >> $wifischedule_section_name
			;;
			"init")
				echo "	option $uci_token_wifi_schedule '1'" >> $wifischedule_section_name
			;;
			"rules")
				var="$(uci show wirelessAvailability | grep timeStart)"
				if [ "$var" == "" ]; then
					echo "There is no rule for wifi schedule!" > /dev/ttyS0
				else
					uci delete wirelessAvailability.reboot
					uci delete wirelessAvailability.wireless
					uci export wirelessAvailability > $tmp_files
					cat $tmp_files | sed '1,1d' | sed 's/ath1/ath01/g' | \
					sed 's/ath2/ath02/g' | sed 's/ath3/ath03/g' | \
					sed 's/ath4/ath04/g' | sed 's/ath5/ath05/g' | \
					sed 's/ath6/ath06/g' | sed 's/ath7/ath07/g' | \
					sed 's/ath050/ath1/g' | sed 's/ath051/ath11/g' | \
					sed 's/ath052/ath12/g' | sed 's/ath053/ath13/g' | \
					sed 's/ath054/ath14/g' | sed 's/ath055/ath15/g' | \
					sed 's/ath056/ath16/g' | sed 's/ath057/ath17/g' >> $wifischedule_section_name
					rm -f $tmp_files
					uci revert wirelessAvailability.reboot
					uci revert wirelessAvailability.wireless
				fi
			;;
		esac
	}
done



#/etc/config/network
echo "Converting network config ........" > /dev/console
#config section
network_section_title="sys loopback lan guest"

#config option
network_section_1="WLANVLANEnable ManagementVLANID"
network_section_2="ifname proto ipaddr netmask"
network_section_3="ifname type proto ipaddr netmask gateway dns ip6LinkLocalEn ip6addr orig_ip6addr ip6gw accept_ra stp hellotime maxage fwdelay priority macaddr"
network_section_4="type proto ipaddr netmask defaultroute peerdns stp gateway dns"
counter=1

for section_token_network in $network_section_title; do
	case $section_token_network in
		"sys")
			echo "config status 'sys'" > $network_section_name		
		;;
		*)
			echo "config interface '$section_token_network'" >> $network_section_name
		;;
	esac

	case $counter in
		1)
			network_section=$network_section_1
		;;
		2)
			network_section=$network_section_2
		;;
		3)
			network_section=$network_section_3
		;;
		4)
			network_section=$network_section_4
		;;
		*)
			network_section=$network_section_1
		;;									
	esac

	for uci_token_network in $network_section; do
		temp_value="$(uci show network.$section_token_network | grep -w "$uci_token_network" | awk -F"=" '{ print $2 }')"
		case $uci_token_network in
			"ManagementVLANID")
				if [ "$temp_value" == "4096" ]; then
					echo "	option $uci_token_network '4094'" >> $network_section_name
				else
					echo "	option $uci_token_network '$temp_value'" >> $network_section_name
				fi
			;;			
			"hellotime")
				echo "	option hello_time '$temp_value'" >> $network_section_name
			;;
			"maxage")
				echo "	option max_age '$temp_value'" >> $network_section_name
			;;
			"fwdelay")
				echo "	option forward_delay '$temp_value'" >> $network_section_name
			;;			
			*)
				[ ! -z "$temp_value" ] && echo "	option $uci_token_network '$temp_value'" >> $network_section_name || 
				{
					case $uci_token_network in
						"WLANVLANEnable")
							var="$(uci get network.0.type)"
							if [ "$var" == "" ]; then
								echo "	option $uci_token_network '0'" >> $network_section_name
							else
								echo "  option $uci_token_network '1'" >> $network_section_name
							fi
						;;
						"ip6addr")
							echo "	option $uci_token_network ''" >> $network_section_name
						;;
						"orig_ip6addr")
							echo "	option $uci_token_network ''" >> $network_section_name
						;;
						"ip6gw")
							echo "	option $uci_token_network ''" >> $network_section_name
						;;												
					esac
				}
			;;		
		esac
	done
	counter=$((counter+1))
	echo "" >> $network_section_name
done

# network VLAN setting for ews500ap / ews510ap
[ $model_name == "ews500ap" ] || [ $model_name == "ews510ap" ] && {
	for vidx in $(seq 0 3)
	do
		echo "config 'interface' 'lan_index${vidx}'" >> $network_section_name
		echo "	option 'enable' '$(uci get network.lan_index${vidx}.enable)'" >> $network_section_name
		echo "	option 'enableVLAN' '$(uci get network.lan_index${vidx}.enableVLAN)'" >> $network_section_name
		echo "	option 'vlanId' '$(uci get network.lan_index${vidx}.vlanId)'" >> $network_section_name
		echo "" >> $network_section_name
	done
}

#config for interface: 0 to 23 & 99
for sitenu in $(seq 0 24)
do
	if [ $sitenu == 0 ]; then
		echo "config interface '99'" >> $network_section_name
	else
		echo "config interface '$((sitenu-1))'" >> $network_section_name
	fi
	if [ "$(uci get network.$sitenu.ifname 2>/dev/null)" != "" ]; then
		echo "	option ifname '$(uci get network.$sitenu.ifname)'" >> $network_section_name;
	else
		echo "	option ifname 'na'" >> $network_section_name
	fi
	echo "	option type 'bridge'" >> $network_section_name
	echo "	option proto 'none'" >> $network_section_name
	echo "" >> $network_section_name
done



#/etc/config/snmpd
echo "Converting snmpd config ........" > /dev/console

#section agent
echo "config 'agent'" > $snmpd_section_name
echo "	option address '$(uci get snmpd.@agent[0].agentaddress)'" >> $snmpd_section_name
echo "" >> $snmpd_section_name

#section snmpv3
snmpd_snmpv3_table="snmpv3Enable userName authProtocol authKey privProtocol privKey engineID"
echo "config 'snmpv3'" >> $snmpd_section_name
for uci_token_snmpd_snmpv3 in $snmpd_snmpv3_table; do
	case $uci_token_snmpd_snmpv3 in
		"snmpv3Enable")
			var="$(uci get snmpd.snmpv3.snmpv3enable)"
			echo "	option $uci_token_snmpd_snmpv3 '$var'" >> $snmpd_section_name
		;;
		"userName")
			var="$(uci get snmpd.snmpv3.username | sed s/\'/$sp_str/g)"
			echo "	option $uci_token_snmpd_snmpv3 '$var'" >> $snmpd_section_name
		;;
		"authProtocol")
			var="$(uci get snmpd.snmpv3.authprotocol)"
			echo "	option $uci_token_snmpd_snmpv3 '$var'" >> $snmpd_section_name
		;;
		"authKey")
			var="$(uci get snmpd.snmpv3.authkey | sed s/\'/$sp_str/g)"
			echo "	option $uci_token_snmpd_snmpv3 '$var'" >> $snmpd_section_name
		;;
		"privProtocol")
			var="$(uci get snmpd.snmpv3.privprotocol)"
			echo "	option $uci_token_snmpd_snmpv3 '$var'" >> $snmpd_section_name
		;;
		"privKey")
			var="$(uci get snmpd.snmpv3.privkey | sed s/\'/$sp_str/g)"
			echo "	option $uci_token_snmpd_snmpv3 '$var'" >> $snmpd_section_name
		;;
		"engineID")
			var="$(uci get snmpd.snmpv3.EngineID | sed s/\'/$sp_str/g)"
			echo "	option $uci_token_snmpd_snmpv3 '$var'" >> $snmpd_section_name
		;;
	esac
done
echo "" >> $snmpd_section_name

#section public/private
snmpd_common_table_1="public private"
snmpd_common_table_1_1="secname source community"
for common_token in $snmpd_common_table_1; do
	echo "config 'com2sec' '$common_token'" >> $snmpd_section_name
	for uci_token_snmpd_common in $snmpd_common_table_1_1; do
		temp_value="$(uci get snmpd."${common_token}"."${uci_token_snmpd_common}" | sed s/\'/$sp_str/g )"
		[ ! -z "$temp_value" ] && echo "	option $uci_token_snmpd_common '$temp_value'" >> $snmpd_section_name
	done
	echo "" >> $snmpd_section_name
done

#section public_v1/public_v2c/public_usm/private_v1/private_v2c/private_usm
snmpd_common_table_2="public_v1 public_v2c public_usm private_v1 private_v2c private_usm"
snmpd_common_table_2_1="group version secname"
for common_token in $snmpd_common_table_2; do
	echo "config 'group' '$common_token'" >> $snmpd_section_name
	for uci_token_snmpd_common in $snmpd_common_table_2_1; do
		case $uci_token_snmpd_common in
			"group")
				temp_value="$(uci show snmpd."${common_token}" | grep -w "${uci_token_snmpd_common}" | awk -F"=" '{ print $2 }' | sed 1d)"
				[ ! -z "$temp_value" ] && echo "	option $uci_token_snmpd_common '$temp_value'" >> $snmpd_section_name
			;;	
			*)
				temp_value="$(uci show snmpd."${common_token}" | grep -w "${uci_token_snmpd_common}" | awk -F"=" '{ print $2 }')"
				[ ! -z "$temp_value" ] && echo "	option $uci_token_snmpd_common '$temp_value'" >> $snmpd_section_name
			;;
		esac
	done
	echo "" >> $snmpd_section_name
done

#section all
snmpd_all_table="viewname type oid"
echo "config 'view' 'all'" >> $snmpd_section_name
for uci_token_snmpd_all in $snmpd_all_table; do
	temp_value="$(uci show snmpd.all | grep -w "${uci_token_snmpd_all}" | awk -F"=" '{ print $2 }')"
	[ ! -z "$temp_value" ] && echo "	option $uci_token_snmpd_all '$temp_value'" >> $snmpd_section_name	
done
echo "" >> $snmpd_section_name

#section public_access/private_access
snmpd_common_table_3="public_access private_access"
snmpd_common_table_3_1="group context version level prefix read write notify"
for common_token in $snmpd_common_table_3; do
	echo "config 'access' '$common_token'" >> $snmpd_section_name
	for uci_token_snmpd_common in $snmpd_common_table_3_1; do
		case $uci_token_snmpd_common in
			*)
				temp_value="$(uci show snmpd."${common_token}" | grep -w "${uci_token_snmpd_common}" | awk -F"=" '{ print $2 }')"
				[ ! -z "$temp_value" ] && echo "	option $uci_token_snmpd_common '$temp_value'" >> $snmpd_section_name
			;;
		esac
	done
	echo "" >> $snmpd_section_name
done

#section system
snmpd_system_table="sysEnable sysLocation sysContact sysName sysDescr"
echo "config 'system'" >> $snmpd_section_name
for uci_token_snmpd_system in $snmpd_system_table; do
	case $uci_token_snmpd_system in
		"sysEnable")
			var="$(uci get snmpd.snmpv2.SnmpEnable)"			
			[ ! -z "$var" ] &&
			echo "	option $uci_token_snmpd_system '$var'" >> $snmpd_section_name ||
			echo "	option $uci_token_snmpd_system '0'" >> $snmpd_section_name
		;;
		*)
			temp_value="$(uci get snmpd.snmpv2."${uci_token_snmpd_system}" | sed s/\'/$sp_str/g)"
			[ ! -z "$temp_value" ] && echo "	option $uci_token_snmpd_system '$temp_value'" >> $snmpd_section_name || 
			{
				case $uci_token_snmpd_system in
					"sysLocation"|"sysContact")
						echo "	option $uci_token_snmpd_system ''" >> $snmpd_section_name
					;;
				esac                        	
			}
		;;	
	esac
done
echo "" >> $snmpd_section_name

#section trap
snmpd_trap_table="destIpAddr destPort community"
echo "config 'trap'" >> $snmpd_section_name
for uci_token_snmpd_trap in $snmpd_trap_table; do
	temp_value="$(uci show snmpd.snmpv2 | grep -w "${uci_token_snmpd_trap}" | awk -F"=" '{ print $2 }')"
	[ ! -z "$temp_value" ] && echo "	option $uci_token_snmpd_trap '$temp_value'" >> $snmpd_section_name ||
	{
		case $uci_token_snmpd_trap in
			"destIpAddr")
				var="$(uci get snmpd.snmpv2.SnmpTrapDstIP)"
				[ ! -z "$var" ] &&
					echo "	option $uci_token_snmpd_trap '$var'" >> $snmpd_section_name ||
					echo "	option $uci_token_snmpd_trap ''" >> $snmpd_section_name
			;;
			"destPort")
				var="$(uci get snmpd.snmpv2.SnmpTrapDstPort)"
				[ ! -z "$var" ] &&
					echo "	option $uci_token_snmpd_trap '$var'" >> $snmpd_section_name ||
					echo "	option $uci_token_snmpd_trap '162'" >> $snmpd_section_name
			;;
			"community")
				var="$(uci get snmpd.snmpv2.SnmpTrapCmnty | sed s/\'/$sp_str/g)"
				[ ! -z "$var" ] &&
					echo "	option $uci_token_snmpd_trap '$var'" >> $snmpd_section_name ||
					echo "	option $uci_token_snmpd_trap 'public'" >> $snmpd_section_name
			;;
		esac
	}
done
echo "" >> $snmpd_section_name

#section exec
snmpd_exec_table="name prog args"
echo "config 'exec'" >> $snmpd_section_name
for uci_token_snmpd_exec in $snmpd_exec_table; do
	temp_value="$(uci show snmpd.@exec[0] | grep -w "${uci_token_snmpd_exec}" | awk -F"=" '{ print $2 }')"
	[ ! -z "$temp_value" ] && echo "	option $uci_token_snmpd_exec '$temp_value'" >> $snmpd_section_name
done



#/etc/config/systime
echo "Converting systime config ........" > /dev/console
systime_table="manual_enable,year,month,day,hour,minute"
uci_token="$(echo $systime_table | sed 's/,/ /g')"
echo "config 'systime' 'systime'" > $systime_section_name
#get senaowrt dut time
tmpdate=$(date +'%m%d%H%M%Y')
month=${tmpdate:0:2}
date=${tmpdate:2:2}
hour=${tmpdate:4:2}
min=${tmpdate:6:2}
year=${tmpdate:8:4}
for uci_token_systime in $uci_token; do
	case $uci_token_systime in
		"manual_enable")
			var="$(uci get ntpclient.@ntpclient[0].TimeSettingMode)"
			if [ "$var" == "1" ]; then
				echo "	option $uci_token_systime '0'" >> $systime_section_name				
			else
				echo "	option $uci_token_systime '1'" >> $systime_section_name
			fi
		;;
		"year")
			echo "	option $uci_token_systime '$year'" >> $systime_section_name
		;;
		"month")
			echo "	option $uci_token_systime '$month'" >> $systime_section_name
		;;
		"day")
			echo "	option $uci_token_systime '$date'" >> $systime_section_name
		;;
		"hour")
			echo "	option $uci_token_systime '$hour'" >> $systime_section_name
		;;
		"minute")
			echo "	option $uci_token_systime '$min'" >> $systime_section_name
		;;               
	esac
done



#Use /rom/etc/config/sysProductInfo
#/etc/config/sysProductInfo
#echo "Converting sysProductInfo config ........" > /dev/console
#section model
#sysProductInfo_model_table="modelName,description,venderName,firmwareVersion,outdoor,fccDfsCertified,etsiDfsCertified"
#sysProductInfo_section_name="/tmp/convert/sysProductInfo"
#uci_token="$(echo $sysProductInfo_model_table | sed 's/,/ /g')"
#echo "config 'model' 'model'" > $sysProductInfo_section_name
#for uci_token_model in $uci_token; do
#	case $uci_token_model in
#		"modelName")
#			echo "	option $uci_token_model '$upper_modelname'" >> $sysProductInfo_section_name
#		;;
#		"description")
#			echo "	option $uci_token_model 'Dual Radio AP, 2T2R, 300Mbps + 300Mbps'" >> $sysProductInfo_section_name
#		;;
#		"venderName")
#			echo "	option $uci_token_model 'EnGenius'" >> $sysProductInfo_section_name
#		;;
#		"firmwareVersion")
#			var="$(cat /etc/version | awk -F" " '{ print $1 }')"
#			echo "	option $uci_token_model '$var'" >> $sysProductInfo_section_name
#		;;
#		"outdoor")
#			echo "	option $uci_token_model '0'" >> $sysProductInfo_section_name
#		;;
#		"fccDfsCertified"|"etsiDfsCertified")
#			echo "	option $uci_token_model '0'" >> $sysProductInfo_section_name
#		;;
#	esac
#done

#echo "" >> $sysProductInfo_section_name
#sysProductInfo_version_table="kernel,application"
#uci_token="$(echo $sysProductInfo_version_table | sed 's/,/ /g')"
#echo "config 'vesrion' 'version'" >> $sysProductInfo_section_name
#for uci_token_version in $uci_token; do
#	case $uci_token_version in
#		"kernel")
#			echo "	option $uci_token_version '3.3.8'" >> $sysProductInfo_section_name
#		;;
#		"application")
#			echo "	option $uci_token_version '0.0.0'" >> $sysProductInfo_section_name
#		;;
#	esac
#done



#/etc/config/ntpclient
echo "Converting ntpclient config ........" > /dev/console
ntpclient_table="startWeek,startDay,startHour,endWeek,endDay,endHour,startMonth,endMonth,DayLightEnable"
uci_token="$(echo $ntpclient_table | sed 's/,/ /g')"
echo "config 'daylightsaving' 'daylightsaving'" > $ntpclient_section_name
for uci_token_ntpclient in $uci_token; do
	case $uci_token_ntpclient in
		*)
			temp_value=$(uci show ntpclient.daylightsaving | grep -w "${uci_token_ntpclient}" | awk -F"=" '{ print $2 }')
			[ ! -z "$temp_value" ] && echo "	option $uci_token_ntpclient '$temp_value'" >> $ntpclient_section_name
		;;
	esac
done



#/etc/config/luci
echo "Converting luci config ........" > /dev/console
#section main
luci_main_table="lang,mediaurlbase,resourcebase"
uci_token="$(echo $luci_main_table | sed 's/,/ /g')"
echo "config core main" > $luci_section_name
for uci_token_luci_main in $uci_token; do
	case $uci_token_luci_main in
		"mediaurlbase")
			echo "	option $uci_token_luci_main '/luci-static/web_ECB_FANCY'" >> $luci_section_name
		;;
		*)
			temp_value=$(uci show luci | grep -w "${uci_token_luci_main}" | awk -F"=" '{ print $2 }')
			[ ! -z "$temp_value" ] && echo "	option $uci_token_luci_main '$temp_value'" >> $luci_section_name	
		;;
	esac
done
echo "" >> $luci_section_name

#section flash_keep
luci_flash_table="uci,dropbear,openvpn,passwd,opkg,firewall,uploads,group,hosts,inittab,profile,rc,lighttpd,shell,webpasswd,sysctl,shadow"
uci_token="$(echo $luci_flash_table | sed 's/,/ /g')"
echo "config extern flash_keep" >> $luci_section_name
for uci_token_luci_flash in $uci_token; do
	case $uci_token_luci_flash in
		"uci")
			var="$(uci get luci.flash_keep.uci)"
			[ ! -z "$var" ] && echo "	option $uci_token_luci_flash '$var'" >> $luci_section_name || 
			echo "	option $uci_token_luci_flash '/etc/config'" >> $luci_section_name
		;;
		"uploads")
			echo "	option $uci_token_luci_flash '/lib/uci/upload/'" >> $luci_section_name
		;;
		"group")
			echo "	option $uci_token_luci_flash '/etc/group'" >> $luci_section_name
		;;
		"hosts")
			echo "	option $uci_token_luci_flash '/etc/hosts'" >> $luci_section_name
		;;
		"inittab")
			echo "	option $uci_token_luci_flash '/etc/inittab'" >> $luci_section_name
		;;
		"profile")
			echo "	option $uci_token_luci_flash '/etc/profile'" >> $luci_section_name
		;;
		"rc")
			echo "	option $uci_token_luci_flash '/etc/rc.local'" >> $luci_section_name
		;;
		"lighttpd")
			echo "	option $uci_token_luci_flash '/etc/lighttpd/lighttpd.conf'" >> $luci_section_name
		;;
		"shell")
			echo "	option $uci_token_luci_flash '/etc/shells'" >> $luci_section_name
		;;
		"webpasswd")
			echo "	option $uci_token_luci_flash '/etc/webpasswd'" >> $luci_section_name
		;;
		"sysctl")
			echo "	option $uci_token_luci_flash '/etc/sysctl.conf'" >> $luci_section_name
		;;
		"shadow")
			echo "	option $uci_token_luci_flash '/etc/shadow'" >> $luci_section_name
		;;
		*)
			temp_value=$(uci show luci.flash_keep | grep -w "${uci_token_luci_flash}" | awk -F"=" '{ print $2 }')
			[ ! -z "$temp_value" ] && echo "	option $uci_token_luci_flash '$temp_value'" >> $luci_section_name
		;;
	esac
done
echo "" >> $luci_section_name
echo "config internal languages" >> $luci_section_name
echo "" >> $luci_section_name
echo "config internal themes" >> $luci_section_name
echo "" >> $luci_section_name

#section sauth
luci_sauth_table="sessionpath,sessiontime"
uci_token="$(echo $luci_sauth_table | sed 's/,/ /g')"
echo "config internal sauth" >> $luci_section_name
for uci_token_luci_sauth in $uci_token; do
	case $uci_token_luci_sauth in
		*)
			temp_value=$(uci show luci.sauth | grep -w "${uci_token_luci_sauth}" | awk -F"=" '{ print $2 }')
			[ ! -z "$temp_value" ] && echo "	option $uci_token_luci_sauth '$temp_value'" >> $luci_section_name
		;;	
	esac
done
echo "" >> $luci_section_name

#section ccache
luci_ccache_table="enable"
uci_token="$(echo $luci_ccache_table | sed 's/,/ /g')"
echo "config internal ccache" >> $luci_section_name
for uci_token_luci_ccache in $uci_token; do
	case $uci_token_luci_ccache in
		*)
			temp_value=$(uci show luci.ccache | grep -w "${uci_token_luci_ccache}" | awk -F"=" '{ print $2 }')
			[ ! -z "$temp_value" ] && echo "	option $uci_token_luci_ccache '$temp_value'" >> $luci_section_name
		;;
	esac
done



#/etc/config/lighttpd
echo "Converting lighttpd config ........" > /dev/console
lighttpd_table="enable,forward"
uci_token="$(echo $lighttpd_table | sed 's/,/ /g')"
echo "config 'https' 'https'" > $lighttpd_section_name
for uci_token_lighttpd in $uci_token; do
	case $uci_token_lighttpd in
		*)
			temp_value=$(uci show secureshell.https | grep -w "${uci_token_lighttpd}" | awk -F"=" '{ print $2 }')
			[ ! -z "$temp_value" ] && echo "	option $uci_token_lighttpd '$temp_value'" >> $lighttpd_section_name	
		;;
	esac
done



#/etc/config/fastroaming
echo "Converting fastroaming config ........" > /dev/console
#section 2.4G
fastroaming_table="enable,adv_search"
uci_token="$(echo $fastroaming_table | sed 's/,/ /g')"
echo "config wifi2G wifi0" > $fastroaming_section_name
for uci_token_fastroaming in $uci_token; do
	case $uci_token_fastroaming in
		*)
			temp_value=$(uci show fastroaming.fastroaming_2G | grep -w "${uci_token_fastroaming}" | awk -F"=" '{ print $2 }')
			[ ! -z "$temp_value" ] && echo "	option $uci_token_fastroaming '$temp_value'" >> $fastroaming_section_name
		;;
	esac
done
echo "" >> $fastroaming_section_name
#section 5G
echo "config wifi5G wifi1" >> $fastroaming_section_name
for uci_token_fastroaming in $uci_token; do
	case $uci_token_fastroaming in
		*)
			temp_value=$(uci show fastroaming.fastroaming_5G | grep -w "${uci_token_fastroaming}" | awk -F"=" '{ print $2 }')
			[ ! -z "$temp_value" ] && echo "	option $uci_token_fastroaming '$temp_value'" >> $fastroaming_section_name
		;;
	esac
done



#/etc/config/emailalert
echo "Converting emailalert config ........" > /dev/console
emailalert_table="auth,alert_event_config,smtpaddr,smtpport,enable,from,alert_to,user,password,security,subject"
echo "config emailalert 'email'" > $emailalert_section_name
uci_token="$(echo $emailalert_table | sed 's/,/ /g')"
for uci_token_emailalert in $uci_token; do
	case $uci_token_emailalert in
		*)
			temp_value=$(uci show system.email | grep -w "${uci_token_emailalert}" | awk -F"=" '{ print $2 }')
			[ ! -z "$temp_value" ] && echo "	option $uci_token_emailalert '$temp_value'" >> $emailalert_section_name ||
			{
				case $uci_token_emailalert in
					"enable")
						var="$(uci get system.email.alert_enable)"
						[ ! -z "$var" ] &&
							echo "	option $uci_token_emailalert '$var'" >> $emailalert_section_name ||
							echo "	option $uci_token_emailalert '0'" >> $emailalert_section_name	
					;;
					"subject")
						var="$(uci get system.email.alert_event_config_subject)"
						[ ! -z "$var" ] &&
							echo "	option $uci_token_emailalert '$var'" >> $emailalert_section_name ||
							echo "	option $uci_token_emailalert " >> $emailalert_section_name
					;;
				esac
			}
		;;
	esac
done



#/etc/config/dropbear
echo "Converting dropbear config ........" > /dev/console
dropbear_table="enable,verbose,PasswordAuth,RootPasswordAuth,Port"
echo "config dropbear" > $dropbear_section_name
uci_token="$(echo $dropbear_table | sed 's/,/ /g')"
for uci_token_dropbear in $uci_token; do
	case $uci_token_dropbear in
		"enable")
			temp_value=$(uci get secureshell.ssh.enable)
			if [ "$temp_value" == "1" ]; then
				temp_value="on"
			else
				temp_value="off"
			fi
			[ ! -z "$temp_value" ] &&
				echo "	option $uci_token_dropbear '$temp_value'" >> $dropbear_section_name ||
				echo "	option $uci_token_dropbear 'off'" >> $dropbear_section_name		
		;;
		*)
			temp_value=$(uci show dropbear | grep -w "${uci_token_dropbear}" | awk -F"=" '{ print $2 }')
			[ ! -z "$temp_value" ] && echo "	option $uci_token_dropbear '$temp_value'" >> $dropbear_section_name ||
			{
				case $uci_token_dropbear in
					"verbose")
						echo "	option $uci_token_dropbear 'on'" >> $dropbear_section_name
					;;
					"RootPasswordAuth")
						echo "	option $uci_token_dropbear 'on'" >> $dropbear_section_name
					;;
				esac
			}
		;;
	esac	
done



#/etc/config/dhcp6c
echo "Converting dhcp6c config ........" > /dev/console
#section basic
dhcp6c_basic_table_1="enabled,interface,dns,debug,pd,na,rapid_commit,domain_name_servers,domain_name,ntp_servers"
dhcp6c_basic_table_2="sip_server_address,sip_server_domain_name,nis_server_address,nis_domain_name,nisp_server_address"
dhcp6c_basic_table_3="nisp_domain_name,bcmcs_server_address,bcmcs_server_domain_name,script"
echo "config 'dhcp6c' 'basic'" > $dhcp6c_section_name
uci_token="$(echo $dhcp6c_basic_table_1 $dhcp6c_basic_table_2 $dhcp6c_basic_table_3 | sed 's/,/ /g')"
for uci_token_basic in $uci_token; do
	case $uci_token_basic in
		*)
			temp_value=$(uci show dhcp6c.basic | grep -w "${uci_token_basic}" | awk -F"=" '{ print $2 }')
			[ ! -z "$temp_value" ] && echo "	option $uci_token_basic '$temp_value'" >> $dhcp6c_section_name
		;;
	esac
done
echo "" >> $dhcp6c_section_name

#section loopback
dhcp6c_loopback_table="enabled,sla_id,sla_len"
echo "config 'interface' 'loopback'" >> $dhcp6c_section_name
uci_token="$(echo $dhcp6c_loopback_table | sed 's/,/ /g')"
for uci_token_loopback in $uci_token; do
	case $uci_token_loopback in
		*)
			temp_value=$(uci show dhcp6c.loopback | grep -w "${uci_token_loopback}" | awk -F"=" '{ print $2 }')
			[ ! -z "$temp_value" ] && echo "	option $uci_token_loopback '$temp_value'" >> $dhcp6c_section_name
		;;
	esac
done
echo "" >> $dhcp6c_section_name

#section lan
dhcp6c_lan_table="enabled,sla_id,sla_len"
echo "config 'interface' 'lan'" >> $dhcp6c_section_name
uci_token="$(echo $dhcp6c_lan_table | sed 's/,/ /g')"
for uci_token_lan in $uci_token; do
	case $uci_token_lan in
		*)
			temp_value=$(uci show dhcp6c.lan | grep -w "${uci_token_lan}" | awk -F"=" '{ print $2 }')
			[ ! -z "$temp_value" ] && echo "	option $uci_token_lan '$temp_value'" >> $dhcp6c_section_name
		;;
	esac
done



#/etc/config/dhcp
echo "Converting dhcp config ........" > /dev/console
#section dnsmasq
dhcp_dnsmasq_table_1="domainneeded boguspriv filterwin2k localise_queries rebind_protection rebind_localhost local domain"
dhcp_dnsmasq_table_2="expandhosts nonegcache authoritative readethers leasefile nonwildcard"
echo "config dnsmasq" > $dhcp_section_name
uci_token="$(echo $dhcp_dnsmasq_table_1 $dhcp_dnsmasq_table_2)"
for uci_token_dnsmasq in $uci_token; do
	temp_value=$(uci show dhcp | grep -w "${uci_token_dnsmasq}" | awk -F"=" '{ print $2 }')
	[ -n "$temp_value" ] && echo "	option $uci_token_dnsmasq '$temp_value'" >> $dhcp_section_name ||
	{
		case $uci_token_dnsmasq in
			"rebind_protection")
				echo "	option $uci_token_dnsmasq 0" >> $dhcp_section_name
			;;
			"rebind_localhost"|"nonwildcard")
				echo "	option $uci_token_dnsmasq 1" >> $dhcp_section_name
			;;
			"boguspriv")
				var="$(uci get dhcp.@dnsmasq[0].boguspriv)"
				[ ! -z "$var" ] &&
					echo "	option $uci_token_dnsmasq '$var'" >> $dhcp_section_name ||
					echo "	option $uci_token_dnsmasq '1'" >> $dhcp_section_name
			;;
		esac
	}	
done
echo "	option resolvfile '/tmp/resolv.conf.auto'" >> $dhcp_section_name
echo "" >> $dhcp_section_name

#section lan
dhcp_lan_table="interface start limit leasetime ignore"
echo "config dhcp 'lan'" >> $dhcp_section_name
for uci_token_lan in $dhcp_lan_table; do
	case $uci_token_lan in
		"start")
			temp_value=$(uci show dhcp.lan | grep -w "${uci_token_lan}" | awk -F"=" '{ print $2 }' | awk -F"." '{ print $4 }')
			[ ! -z "$temp_value" ] && echo "	option $uci_token_lan '$temp_value'" >> $dhcp_section_name
		;;
		*)
			temp_value=$(uci show dhcp.lan | grep -w "${uci_token_lan}" | awk -F"=" '{ print $2 }')			
			[ ! -z "$temp_value" ] && echo "	option $uci_token_lan '$temp_value'" >> $dhcp_section_name
		;;
	esac        
done
echo "" >> $dhcp_section_name

#section wan
dhcp_wan_table="interface ignore"
echo "config dhcp 'wan'" >> $dhcp_section_name
for uci_token_wan in $dhcp_wan_table; do
	temp_value=$(uci show dhcp.wan | grep -w "${uci_token_wan}" | awk -F"=" '{ print $2 }')
	[ ! -z "$temp_value" ] && echo "	option $uci_token_wan '$temp_value'" >> $dhcp_section_name
done
echo "" >> $dhcp_section_name

#section guest
dhcp_guest_table="interface leasetime start dhcp_option ignore"
echo "config dhcp 'guest'" >> $dhcp_section_name
for uci_token_guest in $dhcp_guest_table; do
	temp_value=$(uci show dhcp.guest | grep -w "${uci_token_guest}" | awk -F"=" '{ print $2 }')
	[ ! -z "$temp_value" ] && echo "	option $uci_token_guest '$temp_value'" >> $dhcp_section_name
done
# "limit" in section guest, SmartOpenWRT v2 need it.
guest_s_addr=$(uci get dhcp.guest.start)
guest_s_addr3=$(echo $guest_s_addr|cut -d'.' -f3)
guest_s_addr4=$(echo $guest_s_addr|cut -d'.' -f4)
guest_p_addr=$(uci get dhcp.guest.dhcp_stop_addr)
guest_p_addr3=$(echo $guest_p_addr|cut -d'.' -f3)
guest_p_addr4=$(echo $guest_p_addr|cut -d'.' -f4)
guest_addr3_diff=$(($guest_p_addr3-$guest_s_addr3))
guest_addr4_diff=$(($guest_p_addr4-$guest_s_addr4))
limit_v2=$(($guest_addr3_diff*256+$guest_addr4_diff))
echo "	option limit '$limit_v2'" >> $dhcp_section_name



#/etc/config/apcontroller
echo "Converting apcontroller config ........" > /dev/console
apcontroller_table="enable,force_ac,disc_int,disc_proxy"
echo "config capwap 'capwap'" > $apcontroller_section_name
uci_token="$(echo $apcontroller_table | sed 's/,/ /g')"
for uci_token_apcontroller in $uci_token; do
	case $uci_token_apcontroller in
		*)
			temp_value=$(uci show apcontroller | grep -w "${uci_token_apcontroller}" | awk -F"=" '{ print $2 }')
			[ ! -z "$temp_value" ] && echo "	option $uci_token_apcontroller '$temp_value'" >> $apcontroller_section_name ||
			{
				case $uci_token_apcontroller in
					"enable"|"disc_proxy")
						echo "	option $uci_token_apcontroller '0'" >> $apcontroller_section_name
					;;
					"force_ac")
						echo "	option $uci_token_apcontroller ''" >> $apcontroller_section_name
					;;
					"disc_int")
						echo "	option $uci_token_apcontroller '10'" >> $apcontroller_section_name
					;;
				esac
			}
		;;
	esac
done



#/etc/config/cfg_cli
echo "Converting cfg_cli config ........" > /dev/console
cli_table="cli_enable"
senaowrt_cli_section_name="cliEnable"
echo "config cli" > $cli_section_name
uci_token="$(echo $cli_table | sed 's/,/ /g')"
for uci_token_cli in $uci_token; do
	case $uci_token_cli in
		*)
			temp_value=$(uci show cli| grep -w "${senaowrt_cli_section_name}" | awk -F"=" '{ print $2 }')
			[ ! -z "$temp_value" ] && echo "	option $uci_token_cli '$temp_value'" >> $cli_section_name ||
			echo "	option $uci_token_cli '1'" >> $cli_section_name
		;;
	esac
done



#/etc/config/wireless
echo "Converting wireless config ........" > /dev/console
wifi_2G_5G_table_wifi1_1="channel_config_status channel channel_config_enable channel_config_list phy hwmode txpower fasthandover_status fasthandover_rssi htmode opmode disabled country rts rate" 
wifi_2G_5G_table_wifi1_2="clientlimits_enable clientlimits_number aggregation_enable aggregation_frame aggregation_byte obeyregpower txchainmask rxchainmask"
wifi_2G_5G_table_wifi1_3="bandsteer bandsteerrssi bandsteerpersent distance ext_channel"
wifi_options="device mode ssid encryption bintval dtim_period frag preamble nawds wds disabled mode_display ifname hidden isolate isolation l2_isolatior vlan_id tc_enabled index nss mcastenhance status fastroamingEnable macfilter allowmaclist denymaclist key_id nasid_enable nasid nasport_enable nasport" 
wifi_wdsap_options="device network mode encryption bintval dtim_period frag preamble nawds nawds_encr wds disabled mode_display ifname isolate isolation l2_isolatior nss key_id" 
wifi_guest_options="device network mode encryption bintval dtim_period frag preamble nawds wds disabled mode_display mode_display_2 ifname hidden isolate l2_isolatior tc_enabled nss mcastenhance ssid status" 

set_wps_config() {
	echo "	option wps_pbc '0'" >> $wifi_section_name
	echo "	option wps_label '0'" >> $wifi_section_name
	echo "	option wps_display '0'" >> $wifi_section_name
	echo "	option wps_keypad '0'" >> $wifi_section_name
	echo "	option wps_ext_registrar '0'" >> $wifi_section_name
	echo "	option wps_state '1'" >> $wifi_section_name
	echo "	option wps_disabled_pin '0'" >> $wifi_section_name
	echo "	option wps_manufacturer 'Senao Technologies, Inc.'" >> $wifi_section_name
	echo "	option wps_friendly_name 'Wi-Fi AP'" >> $wifi_section_name
	echo "	option wps_manufacturer_url 'http://www.engeniustech.com/'" >> $wifi_section_name
	echo "	option wps_model_name 'Engenius Wi-Fi AP'" >> $wifi_section_name
	echo "	option wps_model_number '$upper_modelname'" >> $wifi_section_name
	case $model_name in
		"enh1750ext")
			echo "	option wps_device_name 'Engenius 802.11AC AP'" >> $wifi_section_name
			echo "	option wps_model_description '802.11ac Dual-Band Wi-Fi AP'" >> $wifi_section_name
		;;
		"ews310ap"|"ews510ap")
			echo "	option wps_device_name 'Engenius 802.11A/N AP'" >> $wifi_section_name
			echo "	option wps_model_description '802.11abgn Dual-Band Wi-Fi AP'" >> $wifi_section_name
		;;
		"ews500ap")
                        echo "  option wps_device_name 'Engenius 802.11A/N AP'" >> $wifi_section_name
                        echo "  option wps_model_description '802.11abgn Single-Band Wi-Fi AP'" >> $wifi_section_name
                ;;
		*)
			echo "	option wps_device_name 'Engenius 802.11A/N AP'" >> $wifi_section_name
			echo "	option wps_model_description '802.11ac Dual-Band Wi-Fi AP'" >> $wifi_section_name
		;;
	esac
	echo "	option wps_model_url 'http://www.engeniustech.com/'" >> $wifi_section_name
	echo "	option wps_pin ''" >> $wifi_section_name
}

set_normal_config() {
	echo "	option bintval '400'" >> $wifi_section_name
	echo "	option dtim_period '2'" >> $wifi_section_name
	echo "	option preamble '0'" >> $wifi_section_name
	echo "	option mcastenhance '2'" >> $wifi_section_name
}

set_wep_keys_config() {
	wifi_radio=$1
	wifi_idx=$2
	key_id_config=$(uci get wireless.w${wifi_radio}_index${wifi_idx}.WepInputMethod)
	for key_no in `seq 1 4`; do
		temp_value="$(uci get wireless.w${wifi_radio}_index${wifi_idx}.key${key_no} | sed s/\'/$sp_str/g)"
		if [ $key_id_config -eq 1 ]; then 
			echo "	option key${key_no} '$temp_value'" >> $wifi_section_name
		elif [ $key_id_config -eq 2 ]; then
			echo "	option key${key_no} 's:${temp_value}'" >> $wifi_section_name
		fi
	done
}

set_wpapsk_config() {
	echo "--- set wpa psk converting..." >> /dev/console
	psk_options="key wpa_group_rekey"
	wpa_options="auth_server auth_port auth_secret acct_enabled acct_server acct_port acct_secret acct_interval wpa_group_rekey"
	wifi_radio=$1
	wifi_idx=$2
	en_type=$3
	wds_mode="0"

	# Note: For WDS AP(16) / WDS STA(19)
	[ "$wifi_idx" == "16" ] || [ "$wifi_idx" == "19" ] && wds_mode="1"

	[ "$en_type" == "psk" ] && {
		for uci_radio_token in $psk_options; do
			case $uci_radio_token in
				"key")
					tmp_config="$(uci get wireless.w${wifi_radio}_index${wifi_idx}.key | sed s/\'/$sp_str/g)"
					[ ! -z "$tmp_config" ] && echo "	option $uci_radio_token '$tmp_config'" >> $wifi_section_name
				;;
				"wpa_group_rekey")
					tmp_config="$(uci get wireless.w${wifi_radio}_index${wifi_idx}.WLANWpaGroupKeyUpdateInterval)"
					[ ! -z "$tmp_config" ] && echo "	option $uci_radio_token '$tmp_config'" >> $wifi_section_name
				;;
			esac
		done
	}

	[ "$en_type" == "wpa" ] && [ "$wds_mode" == "0" ] && {
		for uci_radio_token in $wpa_options; do
			case $uci_radio_token in
				"wpa_group_rekey")
					tmp_config="$(uci get wireless.w${wifi_radio}_index${wifi_idx}.WLANWpaGroupKeyUpdateInterval)"
					[ ! -z "$tmp_config" ] && echo "	option $uci_radio_token '$tmp_config'" >> $wifi_section_name
				;;
				"auth_server")
					tmp_config="$(uci get wireless.w${wifi_radio}_index${wifi_idx}.server)"
					[ ! -z "$tmp_config" ] && echo "	option $uci_radio_token '$tmp_config'" >> $wifi_section_name
				;;
				"auth_port")
					tmp_config="$(uci get wireless.w${wifi_radio}_index${wifi_idx}.port)"
					[ ! -z "$tmp_config" ] && echo "	option $uci_radio_token '$tmp_config'" >> $wifi_section_name
				;;
				"auth_secret")
					tmp_config="$(uci get wireless.w${wifi_radio}_index${wifi_idx}.WLANWpaRadiusSrvSecret | sed s/\'/$sp_str/g)"
					[ ! -z "$tmp_config" ] && echo "	option $uci_radio_token '$tmp_config'" >> $wifi_section_name
				;;
				"acct_enabled")
					tmp_config="$(uci get wireless.w${wifi_radio}_index${wifi_idx}.WLANWpaAccEnable)"
					[ ! -z "$tmp_config" ] && echo "	option $uci_radio_token '$tmp_config'" >> $wifi_section_name
				;;
				"acct_server")
					tmp_config="$(uci get wireless.w${wifi_radio}_index${wifi_idx}.WLANWpaRadiusAccSrvIP)"
					[ ! -z "$tmp_config" ] && echo "	option $uci_radio_token '$tmp_config'" >> $wifi_section_name
				;;
				"acct_port")
					tmp_config="$(uci get wireless.w${wifi_radio}_index${wifi_idx}.WLANWpaRadiusAccSrvPort)"
					[ ! -z "$tmp_config" ] && echo "	option $uci_radio_token '$tmp_config'" >> $wifi_section_name
				;;
				"acct_secret")
					tmp_config="$(uci get wireless.w${wifi_radio}_index${wifi_idx}.WLANWpaRadiusAccSrvSecret | sed s/\'/$sp_str/g)"
					[ ! -z "$tmp_config" ] && echo "	option $uci_radio_token '$tmp_config'" >> $wifi_section_name
				;;
				"acct_interval")
					tmp_config="$(uci get wireless.w${wifi_radio}_index${wifi_idx}.WLANWpaRadiusAccInterimInterval)"
					[ ! -z "$tmp_config" ] && echo "	option $uci_radio_token '$tmp_config'" >> $wifi_section_name
				;;
			esac
		done
	}
}

# For WDS STA confilg only
set_wdssta_wpapsk_config(){
	radio="$1"

	echo "	option auth '$(uci get wireless.w${radio}_index19.auth)'" >> $wifi_section_name
	echo "	option identity '$(uci get wireless.w${radio}_index19.identity)'" >> $wifi_section_name
	echo "	option password '$(uci get wireless.w${radio}_index19.password)'" >> $wifi_section_name

	# upper case(PEAP/TTLS) convert to lower case(peap/ttls)
	eaptype="$(echo $(uci get wireless.w0_index19.eap_type)| tr 'A-Z' 'a-z')"
	echo "	option eap_type '$eaptype'" >> $wifi_section_name
}

set_security_type_config() {
	case $1 in
		"wep open")
			echo "	option encryption 'wep-open'" >> $wifi_section_name
			set_wep_keys_config $2 $3
		;;
		"wep shared")
			echo "	option encryption 'wep-shared'" >> $wifi_section_name
			set_wep_keys_config $2 $3
		;;
		"psk aes")
			echo "	option encryption 'psk+ccmp'" >> $wifi_section_name
			set_wpapsk_config $2 $3 "psk"
		;;
		"psk tkip")
			echo "	option encryption 'psk+tkip'" >> $wifi_section_name
			set_wpapsk_config $2 $3 "psk"
		;;
		"psk tkip+aes")
			echo "	option encryption 'psk+tkip+ccmp'" >> $wifi_section_name
			set_wpapsk_config $2 $3 "psk"
		;;
		"psk2 aes")
			echo "	option encryption 'psk2+ccmp'" >> $wifi_section_name
			set_wpapsk_config $2 $3 "psk"
		;;
		"psk2 tkip")
			echo "	option encryption 'psk2+tkip'" >> $wifi_section_name
			set_wpapsk_config $2 $3 "psk"
		;;
		"psk2 tkip+aes")
			echo "	option encryption 'psk2+tkip+ccmp'" >> $wifi_section_name
			set_wpapsk_config $2 $3 "psk"
		;;
		"psk-mixed aes")
			echo "	option encryption 'psk-mixed+ccmp'" >> $wifi_section_name
			set_wpapsk_config $2 $3 "psk"
		;;
		"psk-mixed tkip")
			echo "	option encryption 'psk-mixed+tkip'" >> $wifi_section_name
			set_wpapsk_config $2 $3 "psk"
		;;
		"psk-mixed tkip+aes")
			echo "	option encryption 'psk-mixed+tkip+ccmp'" >> $wifi_section_name
			set_wpapsk_config $2 $3 "psk"
		;;
		"wpa aes")
			echo "	option encryption 'wpa+ccmp'" >> $wifi_section_name
			set_wpapsk_config $2 $3 "psk"
		;;
		"wpa tkip")
			echo "	option encryption 'wpa+tkip'" >> $wifi_section_name
			[ $is_support_radius_account -eq 0 ] && set_wpapsk_config $2 $3 "wpa"
			# Note: WDS STA(19)
			[ "$3" == "19" ] && set_wdssta_wpapsk_config $2
		;;
		"wpa tkip+aes")
			echo "	option encryption 'wpa+tkip+ccmp'" >> $wifi_section_name
			[ $is_support_radius_account -eq 0 ] && set_wpapsk_config $2 $3 "wpa"
		;;
		"wpa2 aes")
			echo "	option encryption 'wpa2+ccmp'" >> $wifi_section_name
			[ $is_support_radius_account -eq 0 ] && set_wpapsk_config $2 $3 "wpa"
		;;
		"wpa2 tkip")
			echo "	option encryption 'wpa2+tkip'" >> $wifi_section_name
			[ $is_support_radius_account -eq 0 ] && set_wpapsk_config $2 $3 "wpa"
			# Note: WDS STA(19)
			[ "$3" == "19" ] && set_wdssta_wpapsk_config $2
		;;
		"wpa2 tkip+aes")
			echo "	option encryption 'wpa2+tkip+ccmp'" >> $wifi_section_name
			[ $is_support_radius_account -eq 0 ] && set_wpapsk_config $2 $3 "wpa"
		;;
		"wpa-mixed aes")
			echo "	option encryption 'wpa-mixed+ccmp'" >> $wifi_section_name
			[ $is_support_radius_account -eq 0 ] && set_wpapsk_config $2 $3 "wpa"
		;;
		"wpa-mixed tkip")
			echo "	option encryption 'wpa-mixed+tkip'" >> $wifi_section_name
			[ $is_support_radius_account -eq 0 ] && set_wpapsk_config $2 $3 "wpa"
		;;
		"wpa-mixed tkip+aes")
			echo "	option encryption 'wpa-mixed+tkip+ccmp'" >> $wifi_section_name
			[ $is_support_radius_account -eq 0 ] && set_wpapsk_config $2 $3 "wpa"
		;;
	esac
}

# Note: SenaoWRT WDS BR interface name is w0_index17 / w1_index17
#			SmartOpenWRT WDS BR interface name is wifi0_wds     / wifi1_wds
wds_br_convert(){
	echo "----- section WDS BR converting..." >> /dev/console

	for radio in `seq 0 1`; do
		# fixed uci name
		echo "config wifi-iface 'wifi${radio}_wds'" >> $wifi_section_name
		echo "	option device 'wifi${radio}'" >> $wifi_section_name
		echo "	option network 'lan'" >> $wifi_section_name
		echo "	option mode 'ap'" >> $wifi_section_name
		echo "	option ssid ''" >> $wifi_section_name
		echo "	option encryption 'none'" >> $wifi_section_name
		echo "	option bintval '400'" >> $wifi_section_name
		echo "	option dtim_period '2'" >> $wifi_section_name
		echo "	option frag '2346'" >> $wifi_section_name
		echo "	option preamble '0'" >> $wifi_section_name
		echo "	option nawds '1'" >> $wifi_section_name
		echo "	option wds '$(uci get wireless.w${radio}_index17.wds)'" >> $wifi_section_name
		is_disabled="$(uci get wireless.w${radio}_index17.WLANEnable)"
		[ $is_disabled == "0" ] && is_disabled="1" || is_disabled="0"
		echo "	option disabled '$is_disabled'" >> $wifi_section_name
		echo "	option mode_display 'wds_bridge'" >> $wifi_section_name
		# SmartOpenWRT: wifi0_wds(ath08) / wifi1_wds(ath18)		
		echo "	option ifname 'ath${radio}8'" >> $wifi_section_name
		echo "	option mcastenhance '2'" >> $wifi_section_name
		if [ $radio -eq 1 ]; then
			echo "	option nss '$nss_value'" >> $wifi_section_name
		fi

		# SmartOpenWRT dynamic uci name list below
		echo "	option WLANWDSPeer '$(uci get wireless.w${radio}_index17.WLANWDSPeer)'" >> $wifi_section_name
		echo "	option wlanwdswepkey '$(uci get wireless.w${radio}_index17.WLANWDSWEPKey)'" >> $wifi_section_name
		echo "	option key '$(uci get wireless.w${radio}_index17.WLANWDSAESKey)'" >> $wifi_section_name

		# WDS BR encryption has none / WEP / AES
		#  SenaoWRT uci value:   none / wep / aes
		#  SmartOpenWRT uci:    none / wep / ccmp
		WDSBR_encryption="$(uci get wireless.w${radio}_index17.encryption)"
		case $WDSBR_encryption in
			"aes")
				echo "	option nawds_encr 'ccmp'" >> $wifi_section_name
			;;
			*)
				echo "	option nawds_encr '$WDSBR_encryption'" >> $wifi_section_name
			;;
		esac

		echo "" >> $wifi_section_name
	done
}


# Note: section WDS 2.4G / 5G has (1+4) interfaces
#     < 2.4G WDS AP interface name >
#          SenaoWRT         => w0_index16 (w0_index0 / w0_index1 / w0_index2 / w0_index3)
#          SmartOpenWRT => wifi0_wds_0 (wifi0_wds_1 / wifi0_wds_2 / wifi0_wds_3 / wifi0_wds_4)
#
#     < 5G WDS AP interface name >
#          SenaoWRT         => w1_index16 (w1_index0 / w1_index1 / w1_index2 / w1_index3)
#          SmartOpenWRT => wifi1_wds_0 (wifi1_wds_1 / wifi1_wds_2 / wifi1_wds_3 / wifi1_wds_4)
wds_ap_convert(){
	echo "----- section WDS AP converting..." >> /dev/console

	for radio in `seq 0 1`; do
		# SenaoWRT: w0_index16 / w1_index16
		# SmartOpenWRT: wifi0_wds_0 / wifi1_wds_0
		echo "config wifi-iface 'wifi${radio}_wds_0'" >> $wifi_section_name
		echo "	option device 'wifi${radio}'" >> $wifi_section_name
		echo "	option network 'lan'" >> $wifi_section_name
		echo "	option mode 'ap'" >> $wifi_section_name
		echo "	option ssid ''" >> $wifi_section_name
		echo "	option encryption 'none'" >> $wifi_section_name
		echo "	option bintval '400'" >> $wifi_section_name
		echo "	option dtim_period '2'" >> $wifi_section_name
		echo "	option frag '2346'" >> $wifi_section_name
		echo "	option preamble '0'" >> $wifi_section_name
		echo "	option nawds '1'" >> $wifi_section_name
		echo "	option wds '$(uci get wireless.w${radio}_index16.wds)'" >> $wifi_section_name
		is_disabled="$(uci get wireless.w${radio}_index16.WLANEnable)"
		[ $is_disabled == "0" ] && is_disabled="1" || is_disabled="0"
		echo "	option disabled '$is_disabled'" >> $wifi_section_name
		echo "	option mode_display 'wds_ap'" >> $wifi_section_name
		if [ $radio -eq 0 ]; then
			# SmartOpenWRT: wifi0_wds_0(ath24)
			echo "	option ifname 'ath24'" >> $wifi_section_name
		else
			# SmartOpenWRT: wifi1_wds_0(ath54)
			echo "	option ifname 'ath54'" >> $wifi_section_name
			echo "	option nss '$nss_value'" >> $wifi_section_name
		fi
		# SmartOpenWRT dynamic uci name list below
		echo "	option WLANWDSPeer '$(uci get wireless.w${radio}_index16.WLANWDSPeer)'" >> $wifi_section_name
		echo "	option wlanwdswepkey '$(uci get wireless.w${radio}_index16.WLANWDSWEPKey)'" >> $wifi_section_name
		echo "	option key '$(uci get wireless.w${radio}_index16.WLANWDSAESKey)'" >> $wifi_section_name
		# WDS AP encryption has none / AES
		#  SenaoWRT uci value:   none / aes
		#  SmartOpenWRT uci:    none / ccmp
		WDSAP_encryption="$(uci get wireless.w${radio}_index16.encryption)"
		case $WDSAP_encryption in
			"aes")
				echo "	option nawds_encr 'ccmp'" >> $wifi_section_name
			;;
			*)
				echo "	option nawds_encr 'none'" >> $wifi_section_name
			;;
		esac
		echo "" >> $wifi_section_name


		# SenaoWRT: w0_indexN / w1_indexN, N=0~3
		# SmartOpenWRT: wifi0_wds_N / wifi1_wds_N,  N=1~4
		for idx in `seq 1 4`; do
			echo "config wifi-iface 'wifi${radio}_wds_$idx'" >> $wifi_section_name
			for uci_radio_token in $wifi_options; do
				temp_value="$(uci get wireless.w${radio}_index$((idx-1))."${uci_radio_token}" | sed s/\'/$sp_str/g)"
				[ ! -z "$temp_value" -a "$uci_radio_token" != "ifname" -a "$uci_radio_token" != "index" -a "$uci_radio_token" != "encryption" ] && echo "	option $uci_radio_token '$temp_value'" >> $wifi_section_name || 
				{
					case $uci_radio_token in
						"index")
							if [ $radio -eq 0 ]; then
								# 2.4G four interface's index from 8 to 11
								echo "	option $uci_radio_token '$((idx+7))'" >> $wifi_section_name
							else
								# 5G four interface's index from 20 to 23
								echo "	option $uci_radio_token '$((idx+19))'" >> $wifi_section_name
							fi
						;;
						"disabled")
							tmp_config="$(uci get wireless.w${radio}_index$((idx-1)).WLANEnable)"
							if [ $tmp_config -eq 0 ]; then
								cnv_v='1'
							else
								cnv_v='0'
							fi
							echo "	option $uci_radio_token '$cnv_v'" >> $wifi_section_name
						;;
						"ifname")
							if [ $radio -eq 0 ]; then
								echo "	option $uci_radio_token 'ath2$((idx-1))'" >> $wifi_section_name
							else
								echo "	option $uci_radio_token 'ath5$((idx-1))'" >> $wifi_section_name
							fi
						;;
						"mode_display")
							echo "	option $uci_radio_token 'wds_ap'" >> $wifi_section_name
						;;
						"vlan_id")
							tmp_config="$(uci get wireless.w${radio}_index$((idx-1)).WLANVLANID)"
							echo "	option $uci_radio_token '$tmp_config'" >> $wifi_section_name
						;;
						"isolation")
							tmp_config="$(uci get wireless.w${radio}_index$((idx-1)).WLANVLANEnable)"
							echo "	option $uci_radio_token '$tmp_config'" >> $wifi_section_name
						;;
						"tc_enabled")
							if [ $radio -eq 0 ]; then
								tmp_config="$(uci get eccaptive.ath$((idx-1)).globallimit)"
								tmp_config_1="$(uci get eccaptive.ath$((idx-1)).uploadlimit)"
							else
								tmp_config="$(uci get eccaptive.ath5$((idx-1)).globallimit)"
								tmp_config_1="$(uci get eccaptive.ath5$((idx-1)).uploadlimit)"
							fi
							if [ "$tmp_config" == "-1" ];then
								echo "	option $uci_radio_token '0'" >> $wifi_section_name
							else
								echo "	option $uci_radio_token '1'" >> $wifi_section_name
								echo "	option tc_downlimit '$tmp_config_1'" >> $wifi_section_name
								echo "	option tc_uplimit '$tmp_config'" >> $wifi_section_name
							fi
							echo "	option tc_downperuser '0'" >>$wifi_section_name
							echo "	option tc_upperuser '0'" >> $wifi_section_name
						;;
						"nawds_encr")
							tmp_config="$(uci get wireless.w${radio}_index16.encryption)"
							echo "	option $uci_radio_token '$tmp_config'" >> $wifi_section_name
						;;
						"nawds")
							echo "	option $uci_radio_token '0'" >> $wifi_section_name
						;;
						"nss")
							if [ $radio -eq 1 ]; then
								echo "	option $uci_radio_token '$nss_value'" >> $wifi_section_name
							fi
						;;
						"encryption")
							if [ "$temp_value" == "none" ]; then
								echo "	option encryption 'none'" >> $wifi_section_name
							else
								set_security_type_config "$temp_value" $radio $((idx-1))
							fi
							[ $is_support_radius_account -eq 1 ] && set_wpapsk_config $radio $((idx-1)) "wpa"
						;;
						"key_id")
							tmp_config="$(uci get wireless.w${radio}_index$((idx-1)).WepKeyIdx)" 2>/dev/null
							echo "	option $uci_radio_token '$tmp_config'" >> $wifi_section_name
						;;
					esac
				}
			done

			set_normal_config

			# For wifi0_wds_1 / wifi1_wds_1
			if [ $idx -eq 1 ]; then
				set_wps_config
			fi
			echo "" >> $wifi_section_name	
		done
	done	
}


# Note: SenaoWRT WDS STA interface name is (w0_index19 / w1_index19)
#			SmartOpenWRT WDS STA interface name is (wifi0_wds_sta / wifi1_wds_sta)
wds_sta_convert(){
	echo "----- section WDS STA converting..." >> /dev/console

	for radio in `seq 0 1`; do
		# fixed uci name
		echo "config wifi-iface 'wifi${radio}_wds_sta'" >> $wifi_section_name
		echo "	option device 'wifi${radio}'" >> $wifi_section_name
		echo "	option network 'lan'" >> $wifi_section_name
		echo "	option mode 'sta'" >> $wifi_section_name
		echo "	option ssid '$(uci get wireless.w${radio}_index19.ssid | sed s/\'/$sp_str/g)'" >> $wifi_section_name	
		echo "	option bintval '100'" >> $wifi_section_name
		echo "	option dtim_period '2'" >> $wifi_section_name
		echo "	option frag '2346'" >> $wifi_section_name
		echo "	option preamble '0'" >> $wifi_section_name
		echo "	option nawds '0'" >> $wifi_section_name
		echo "	option wds '$(uci get wireless.w${radio}_index19.wds)'" >> $wifi_section_name
		is_disabled="$(uci get wireless.w${radio}_index19.WLANEnable)"
		[ $is_disabled == "0" ] && is_disabled="1" || is_disabled="0"
		echo "	option disabled '$is_disabled'" >> $wifi_section_name
		echo "	option mode_display 'wds_sta'" >> $wifi_section_name

		# Note: SmartOpenWRT: wifi0_wds_sta(ath25) / wifi1_wds_sta(ath55)
		if [ $radio -eq 0 ]; then
			echo "	option ifname 'ath25'" >> $wifi_section_name
		else
			echo "	option ifname 'ath55'" >> $wifi_section_name
		fi

		# "PreferBSSIDEnable / bssid" is SenaoWRT dynamic uci name.
		wds_prefer="$(uci get wireless.w${radio}_index19.PreferBSSIDEnable)" 2>/dev/null
		echo "	option preferbssid_enable '${wds_prefer:-0}'" >> $wifi_section_name
		wds_bssid="$(uci get wireless.w${radio}_index19.bssid)" 2>/dev/null
		echo "	option bssid '${wds_bssid:-}'" >> $wifi_section_name

		# WDS STA encryption has Disabled / WEP / WPA-PSK / WPA2-PSK / WPA-Enterprise / WPA2-Enterprise
		enc="$(uci get wireless.w${radio}_index19.encryption)"
		if [ "$enc" == "none" ]; then
			echo "	option encryption 'none'" >> $wifi_section_name
		else
			set_security_type_config "$enc" "$radio" "19" "wds"
		fi

		echo "" >> $wifi_section_name
	done
}

# enh1750ext needs this part, otherwise wireless page fail.
# We give all a default value at this stage
cb_convert(){
	echo "config wifi-iface 'wifi0_sta'" >> $wifi_section_name
	echo "	option device 'wifi0'" >> $wifi_section_name
	echo "	option network 'lan'" >> $wifi_section_name
	echo "	option mode 'sta'" >> $wifi_section_name
	echo "	option ssid 'AP SSID'" >> $wifi_section_name
	echo "	option encryption 'none'" >> $wifi_section_name
	echo "	option bintval '100'" >> $wifi_section_name
	echo "	option dtim_period '2'" >> $wifi_section_name
	echo "	option frag '2346'" >> $wifi_section_name
	echo "	option preamble '0'" >> $wifi_section_name
	echo "	option nawds '0'" >> $wifi_section_name
	echo "	option wds '0'" >> $wifi_section_name
	echo "	option disabled '0'" >> $wifi_section_name
	echo "	option mode_display 'sta'" >> $wifi_section_name
	echo "	option ifname 'ath26'" >> $wifi_section_name
	echo "	option preferbssid_enable '0'" >> $wifi_section_name
	echo "	option bssid ''" >> $wifi_section_name
	echo "" >> $wifi_section_name

	echo "config wifi-iface 'wifi1_sta'" >> $wifi_section_name
	echo "	option device 'wifi1'" >> $wifi_section_name
	echo "	option network 'lan'" >> $wifi_section_name
	echo "	option mode 'sta'" >> $wifi_section_name
	echo "	option ssid 'AP SSID_5G'" >> $wifi_section_name
	echo "	option encryption 'none'" >> $wifi_section_name
	echo "	option bintval '100'" >> $wifi_section_name
	echo "	option dtim_period '2'" >> $wifi_section_name
	echo "	option frag '2346'" >> $wifi_section_name
	echo "	option preamble '0'" >> $wifi_section_name
	echo "	option nawds '0'" >> $wifi_section_name
	echo "	option wds '0'" >> $wifi_section_name
	echo "	option disabled '0'" >> $wifi_section_name
	echo "	option mode_display 'sta'" >> $wifi_section_name
	echo "	option ifname 'ath56'" >> $wifi_section_name
	echo "	option preferbssid_enable '0'" >> $wifi_section_name
	echo "	option bssid ''" >> $wifi_section_name
	echo "" >> $wifi_section_name						
}

### wifi1 ###
echo "config wifi-device 'wifi1'" > $wifi_section_name
echo "	option type 'qcawifi'" >> $wifi_section_name
uci_token="$(echo $wifi_2G_5G_table_wifi1_1 $wifi_2G_5G_table_wifi1_2 $wifi_2G_5G_table_wifi1_3)"
for uci_token_5g in $uci_token; do
	temp_value=$(uci show wireless.wifi1 | grep -w "${uci_token_5g}" | awk -F"=" '{ print $2 }')	
	case $uci_token_5g in
		"country")
			if [ "$temp_value" == "0" ]; then
				echo "	option $uci_token_5g '00'" >> $wifi_section_name
			else
				echo "	option $uci_token_5g '$temp_value'" >> $wifi_section_name
			fi
		;;
		"hwmode")
			if [ "$temp_value" == "11nac" -o "$temp_value" == "11naac" ]; then
			    echo "	option $uci_token_5g '11ac'" >> $wifi_section_name
			elif [ "$temp_value" == "11na_pure" ]; then
			    echo "	option $uci_token_5g '11n'" >> $wifi_section_name
			else
				echo "	option $uci_token_5g '$temp_value'" >> $wifi_section_name
			fi
        ;;
        "channel_config_status")
			channel="$(uci get wireless.wifi1.channel)"
			if [ "$channel" == "auto" ]; then
			    echo "	option $uci_token_5g '1'" >> $wifi_section_name
			else
			    echo "	option $uci_token_5g '4'" >> $wifi_section_name
			fi
		;;
		"ext_channel")
			extchannel="$(uci get wireless.wifi1.WLANExtChannel)"
			echo "	option $uci_token_5g '$extchannel'" >> $wifi_section_name
		;;
		"opmode")
			var="$(uci get wireless.wifi1.opmode)"
			case $var in
				"br")
					wdsbr="$(uci get wireless.w1_index17.wds)"
					[ "$wdsbr" == "1" ] && {
						# SmartOpenWrt 5G WDS BR mode
						echo "	option $uci_token_5g 'wds_bridge'" >> $wifi_section_name
					} || echo "	option $uci_token_5g 'br'" >> $wifi_section_name
				;;
				"apwds")
					# WDS AP mode
					echo "	option $uci_token_5g 'wds_ap'" >> $wifi_section_name
				;;
				"stawds")
					# WDS STA mode
					echo "	option $uci_token_5g 'wds_sta'" >> $wifi_section_name
				;;
				*)
					echo "	option $uci_token_5g '${var:-ap}'" >> $wifi_section_name
				;;
			esac
		;;
		*)
			[ ! -z "$temp_value" ] && echo "	option $uci_token_5g '$temp_value'" >> $wifi_section_name || 
			{
				case $uci_token_5g in
					"channel_config_enable")
						echo "	option $uci_token_5g '0'" >> $wifi_section_name
					;;
					"channel_config_list")
						echo "	option $uci_token_5g ''" >> $wifi_section_name
					;;
					"phy")
						echo "	option $uci_token_5g 'phy0'" >> $wifi_section_name
					;; 
					"fasthandover_status")
						var="$(uci get fasthandover.fasthandover.Enable)"
						[ ! -z "$var" ] &&
						echo "	option $uci_token_5g '$var'" >> $wifi_section_name || 
						echo "	option $uci_token_5g '0'" >> $wifi_section_name
					;;
					"fasthandover_rssi")
						var="$(uci get fasthandover.fasthandover.RSSI)"
						[ ! -z "$var" ] &&
						echo "	option $uci_token_5g '$var'" >> $wifi_section_name ||
						echo "	option $uci_token_5g '-85'" >> $wifi_section_name
					;;
					"htmode")
						var="$(uci get wireless.wifi1.WLANHTMode)"
						[ ! -z "$var" ] &&
						{
							if [ "$var" == "20-40" ]; then
								echo "	option $uci_token_5g 'HT20_40'" >> $wifi_section_name
							else
								echo "	option $uci_token_5g 'HT$var'" >> $wifi_section_name
							fi
						} || echo "	option $uci_token_5g 'HT40'" >> $wifi_section_name 
					;;
					"disabled")
						echo "	option $uci_token_5g '0'" >> $wifi_section_name
					;;
					"rts")
						var="$(uci get wireless.w1_index0.rts)"
						[ ! -z "$var" ] &&
						echo "	option $uci_token_5g '$var'" >> $wifi_section_name ||
						echo "	option $uci_token_5g '2436'" >> $wifi_section_name
					;;
					"rate")
						#11AC didn't support data rate in uci
						tmp_htmode="$(uci get wireless.wifi1.hwmode)"
						if [ "$tmp_htmode" != "11ac" -a "$tmp_htmode" != "11naac" ]; then
							var="$(uci get wireless.w1_index0.rate)"
							search_rate $var
							echo "	option $uci_token_5g '$tmp_rate'" >> $wifi_section_name
						fi
                    ;;                                        
					"clientlimits_enable")
						var="$(uci get clientcontrol.clientlimit_5g.enable)"
						[ ! -z "$var" ] &&
						echo "	option $uci_token_5g '$var'" >> $wifi_section_name ||
						echo "	option $uci_token_5g '1'" >> $wifi_section_name
					;;
					"clientlimits_number")
						var="$(uci get clientcontrol.clientlimit_5g.maxclient)"
						[ ! -z "$var" ] &&
						echo "	option $uci_token_5g '$var'" >> $wifi_section_name ||
						echo "	option $uci_token_5g '127'" >> $wifi_section_name
					;;
					"aggregation_enable")
						var="$(uci get wireless.wifi1.WLANAggreEnable)"
						[ ! -z "$var" ] &&
						echo "	option $uci_token_5g '$var'" >> $wifi_section_name ||
						echo "	option $uci_token_5g '1'" >> $wifi_section_name
					;;
					"aggregation_frame")
						var="$(uci get wireless.wifi1.WLANAggreFrames)"
						[ ! -z "$var" ] &&
						echo "	option $uci_token_5g '$var'" >> $wifi_section_name ||
						echo "	option $uci_token_5g '32'" >> $wifi_section_name
					;;
					"aggregation_byte")
						var="$(uci get wireless.wifi1.WLANAggreBytes)"
						[ ! -z "$var" ] &&
						echo "	option $uci_token_5g '$var'" >> $wifi_section_name ||
						echo "	option $uci_token_5g '50000'" >> $wifi_section_name
					;;
					"txchainmask"|"rxchainmask")
						case $nss_value in
							"1")
								echo "	option $uci_token_5g '1'" >> $wifi_section_name
							;;
							"2")
								echo "	option $uci_token_5g '3'" >> $wifi_section_name
							;;
							"3")
								echo "	option $uci_token_5g '7'" >> $wifi_section_name
							;;
						esac
					;;
					"bandsteer")
						var="$(uci get wireless.wifi0.bandsteer)" 2>/dev/null
						[ ! -z "$var" ] &&
						echo "	option $uci_token_5g '$var'" >> $wifi_section_name ||	
						echo "	option $uci_token_5g '0'" >> $wifi_section_name
					;;
					"bandsteerrssi")
						var="$(uci get wireless.wifi0.bandsteerrssi)" 2>/dev/null
						[ ! -z "$var" ] &&
						echo "	option $uci_token_5g '$var'" >> $wifi_section_name ||
						echo "	option $uci_token_5g '21'" >> $wifi_section_name
					;;
					"bandsteerpersent")
						var="$(uci get wireless.wifi0.bandsteerpersent)" 2>/dev/null
						[ ! -z "$var" ] &&
						echo "	option $uci_token_5g '$var'" >> $wifi_section_name ||
						echo "	option $uci_token_5g '75'" >> $wifi_section_name
					;;
				esac
			}
		;;
	esac
done
echo "" >> $wifi_section_name

### wifi0 ###
echo "config wifi-device 'wifi0'" >> $wifi_section_name
echo "	option type 'qcawifi'" >> $wifi_section_name
for uci_token_2g in $uci_token; do
	temp_value=$(uci show wireless.wifi0 | grep -w "${uci_token_2g}" | awk -F"=" '{ print $2 }')
	case $uci_token_2g in
		"country")
			if [ "$temp_value" == "0" ]; then
				echo "	option $uci_token_2g '00'" >> $wifi_section_name
			else
				echo "	option $uci_token_2g '$temp_value'" >> $wifi_section_name
			fi
		;;        	
		"hwmode")
			if [ "$temp_value" == "11nac" ]; then
				echo "	option $uci_token_2g '11ac'" >> $wifi_section_name
			elif [ "$temp_value" == "11na_pure" -o "$temp_value" == "11n_pure" ]; then
				echo "	option $uci_token_2g '11n'" >> $wifi_section_name
			else
				echo "	option $uci_token_2g '$temp_value'" >> $wifi_section_name						
			fi
		;;
		"channel_config_status")
			channel="$(uci get wireless.wifi0.channel)"
			if [ "$channel" == "auto" ]; then
				echo "	option $uci_token_2g '1'" >> $wifi_section_name
			else
				echo "	option $uci_token_2g '4'" >> $wifi_section_name
			fi
		;;
		"ext_channel")
			extchannel="$(uci get wireless.wifi0.WLANExtChannel)"
			echo "	option $uci_token_2g '$extchannel'" >> $wifi_section_name
		;;
		"opmode")
			var="$(uci get wireless.wifi0.opmode)"
			case $var in
				"br")
					wdsbr="$(uci get wireless.w0_index17.wds)"
					[ "$wdsbr" == "1" ] && {
						# SmartOpenWrt 2.4G WDS BR mode
						echo "	option $uci_token_2g 'wds_bridge'" >> $wifi_section_name
					} || echo "	option $uci_token_2g 'br'" >> $wifi_section_name
				;;
				"apwds")
					# WDS AP mode
					echo "	option $uci_token_2g 'wds_ap'" >> $wifi_section_name
				;;
				"stawds")
					# WDS STA mode
					echo "	option $uci_token_2g 'wds_sta'" >> $wifi_section_name
				;;
				*)
					echo "	option $uci_token_2g '${var:-ap}'" >> $wifi_section_name
				;;
			esac
		;;
		*)
			[ ! -z "$temp_value" ] && echo "	option $uci_token_2g '$temp_value'" >> $wifi_section_name ||
			{
				case $uci_token_2g in
					"channel_config_enable")
						echo "	option $uci_token_2g '0'" >> $wifi_section_name
					;;
					"channel_config_list")
						echo "	option $uci_token_2g ''" >> $wifi_section_name
					;;
					"phy")
						echo "	option $uci_token_2g 'phy0'" >> $wifi_section_name
					;;
					"fasthandover_status")
						var="$(uci get fasthandover.fasthandover.Enable)"
						[ ! -z "$var" ] &&
						echo "	option $uci_token_2g '$var'" >> $wifi_section_name ||
						echo "	option $uci_token_2g '0'" >> $wifi_section_name
					;;
					"fasthandover_rssi")
						var="$(uci get fasthandover.fasthandover.RSSI)"
						[ ! -z "$var" ] &&
						echo "	option $uci_token_2g '$var'" >> $wifi_section_name ||
						echo "	option $uci_token_2g '-85'" >> $wifi_section_name
					;;
					"htmode")
						var="$(uci get wireless.wifi0.WLANHTMode)"
						[ ! -z "$var" ] &&
						{
							if [ "$var" == "20-40" ]; then
								echo "	option $uci_token_2g 'HT20_40'" >> $wifi_section_name
							else
								echo "	option $uci_token_2g 'HT$var'" >> $wifi_section_name
							fi
						} || echo "	option $uci_token_2g 'HT40'" >> $wifi_section_name
					;;
					"disabled")
						echo "	option $uci_token_2g '0'" >> $wifi_section_name
					;;
					"rts")
						var="$(uci get wireless.w0_index0.rts)"
						[ ! -z "$var" ] &&
						echo "	option $uci_token_2g '$var'" >> $wifi_section_name ||
						echo "	option $uci_token_2g '2436'" >> $wifi_section_name
					;;
					"rate")
						var="$(uci get wireless.w0_index0.rate)"
						search_rate $var
						echo "	option $uci_token_2g '$tmp_rate'" >> $wifi_section_name
					;;                                                                                
					"clientlimits_enable")
						var="$(uci get clientcontrol.clientlimit_2g.enable)"
						[ ! -z "$var" ] &&
						echo "	option $uci_token_2g '$var'" >> $wifi_section_name ||
						echo "	option $uci_token_2g '1'" >> $wifi_section_name
					;;
					"clientlimits_number")
						var="$(uci get clientcontrol.clientlimit_2g.maxclient)"
						[ ! -z "$var" ] &&
						echo "	option $uci_token_2g '$var'" >> $wifi_section_name ||
						echo "	option $uci_token_2g '127'" >> $wifi_section_name
					;;
					"aggregation_enable")
						var="$(uci get wireless.wifi0.WLANAggreEnable)"
						[ ! -z "$var" ] &&
						echo "	option $uci_token_2g '$var'" >> $wifi_section_name ||
						echo "	option $uci_token_2g '1'" >> $wifi_section_name
					;;
					"aggregation_frame")
						var="$(uci get wireless.wifi0.WLANAggreFrames)"
						[ ! -z "$var" ] &&
						echo "	option $uci_token_2g '$var'" >> $wifi_section_name ||
						echo "	option $uci_token_2g '32'" >> $wifi_section_name
					;;
					"aggregation_byte")
						var="$(uci get wireless.wifi0.WLANAggreBytes)"
						[ ! -z "$var" ] &&
						echo "	option $uci_token_2g '$var'" >> $wifi_section_name ||
						echo "	option $uci_token_2g '50000'" >> $wifi_section_name
					;;
					"txchainmask"|"rxchainmask")
						case $nss_value in
							"1")
								echo "	option $uci_token_2g '1'" >> $wifi_section_name
							;;
							"2")
								echo "	option $uci_token_2g '3'" >> $wifi_section_name
							;;
							"3")
								echo "	option $uci_token_2g '7'" >> $wifi_section_name
							;;
						esac
					;;
					"bandsteer")
						var="$(uci get wireless.wifi0.bandsteer)" 2>/dev/null
						[ ! -z "$var" ] &&
						echo "	option $uci_token_2g '$var'" >> $wifi_section_name ||
						echo "	option $uci_token_2g '0'" >> $wifi_section_name
					;;
					"bandsteerrssi")
						var="$(uci get wireless.wifi0.bandsteerrssi)" 2>/dev/null
						[ ! -z "$var" ] &&
						echo "	option $uci_token_2g '$var'" >> $wifi_section_name ||
						echo "	option $uci_token_2g '21'" >> $wifi_section_name
					;;
					"bandsteerpersent")
						var="$(uci get wireless.wifi0.bandsteerpersent)" 2>/dev/null
						[ ! -z "$var" ] &&
						echo "	option $uci_token_2g '$var'" >> $wifi_section_name ||
						echo "	option $uci_token_2g '75'" >> $wifi_section_name
					;;
				esac
			}
		;;
	esac
done
echo "" >> $wifi_section_name                                        

### 2.4G / 5G has 8 interfaces ###
for radio in `seq 0 1`; do
	for idx in `seq 1 8`; do
		echo "config wifi-iface 'wifi${radio}_ssid_$idx'" >> $wifi_section_name

		# For "network" only.
		# e.g. wireless.w0_index0.network=1  convert to  wireless.wifi0_ssid_1.network=0 (when 2.4G 1th interface active)
		nwk="$(uci get wireless.w${radio}_index$((idx-1)).network)"
		[ "$nwk" == "lan" ] && {
			echo "	option network 'lan'" >> $wifi_section_name
		} || {
			echo "	option network '$((nwk-1))'" >> $wifi_section_name
		}

		for uci_radio_token in $wifi_options; do
			temp_value="$(uci get wireless.w${radio}_index$((idx-1))."${uci_radio_token}" | sed s/\'/$sp_str/g)"
			[ ! -z "$temp_value" -a "$uci_radio_token" != "ifname" -a "$uci_radio_token" != "index" -a "$uci_radio_token" != "encryption" ] && echo "	option $uci_radio_token '$temp_value'" >> $wifi_section_name || 
			{
				case $uci_radio_token in
					"index")
						if [ $radio -eq 0 ]; then
							# 2.4G
							# All SmartOpenWrt model 2.4G eight interface's index from 0 to 7
							echo "	option $uci_radio_token '$((idx-1))'" >> $wifi_section_name
						else
							# 5G
							case $model_name in
								"enh1750ext")
									# SmartOpenWrt ENH1750EXT 5G eight interface's index from 12 to 19 (index 8~11 belongs to WDS interface)
									echo "	option $uci_radio_token '$((idx+11))'" >> $wifi_section_name
								;;
								*)
									# SmartOpenWrt EWS-series AP 5G eight interface's index from 8 to 15
									echo "	option $uci_radio_token '$((idx+7))'" >> $wifi_section_name
								;;
							esac
						fi
					;;
					"disabled")
						tmp_config="$(uci get wireless.w${radio}_index$((idx-1)).WLANEnable)"
						if [ $tmp_config -eq 0 ]; then
							cnv_v='1'
						else
							cnv_v='0'
						fi
						echo "	option $uci_radio_token '$cnv_v'" >> $wifi_section_name
					;;
					"ifname")
						if [ $radio -eq 0 ]; then
							if [ $idx -eq 1 ]; then
								echo "	option $uci_radio_token 'ath0'" >> $wifi_section_name
							else
								echo "	option $uci_radio_token 'ath0$((idx-1))'" >> $wifi_section_name
							fi
						else
							if [ $idx -eq 1 ]; then
								echo "	option $uci_radio_token 'ath1'" >> $wifi_section_name
							else
								echo "	option $uci_radio_token 'ath1$((idx-1))'" >> $wifi_section_name
							fi
						fi
					;;
					"tc_enabled")
						if [ $radio -eq 0 ]; then
							tmp_config="$(uci get eccaptive.ath$((idx-1)).uploadlimit)"
							tmp_config_1="$(uci get eccaptive.ath$((idx-1)).globallimit)"
						else
							tmp_config="$(uci get eccaptive.ath5$((idx-1)).uploadlimit)"
							tmp_config_1="$(uci get eccaptive.ath5$((idx-1)).globallimit)"
						fi
						if [ "$tmp_config" == "-1" ];then
							echo "	option $uci_radio_token '0'" >> $wifi_section_name
							echo "	option tc_downlimit ''" >> $wifi_section_name
							echo "	option tc_uplimit ''" >> $wifi_section_name
						else
							echo "	option $uci_radio_token '1'" >> $wifi_section_name
							echo "	option tc_downlimit '$tmp_config_1'" >> $wifi_section_name
							echo "	option tc_uplimit '$tmp_config'" >> $wifi_section_name
						fi
						echo "	option tc_downperuser '0'" >> $wifi_section_name
						echo "	option tc_upperuser '0'" >> $wifi_section_name	
					;;
					"mode_display")
						echo "	option $uci_radio_token 'ap'" >> $wifi_section_name
					;;
					"nawds")
						echo "	option $uci_radio_token '0'" >> $wifi_section_name
					;;
					"isolation")
						tmp_config="$(uci get wireless.w${radio}_index$((idx-1)).WLANVLANEnable)"
						echo "	option $uci_radio_token '$tmp_config'" >> $wifi_section_name
					;;
					"l2_isolatior")
						tmp_config="$(uci get wireless.w${radio}_index$((idx-1)).L2_isolation)"
						echo "	option $uci_radio_token '$tmp_config'" >> $wifi_section_name
					;;
					"vlan_id")
						tmp_config="$(uci get wireless.w${radio}_index$((idx-1)).WLANVLANID)"
						echo "	option $uci_radio_token '$tmp_config'" >> $wifi_section_name
					;;
					"status")
						echo "	option status '0'" >> $wifi_section_name
					;;
					"fastroamingEnable")
						tmp_enc="$(uci get wireless.w${radio}_index$((idx-1)).encryption | sed s/\'/$sp_str/g)"
						# psk2 aes
						# psk2 tkip+aes
						# psk-mixed aes
						# psk-mixed tkip+aes
						# wpa2 aes
						# wpa2 tkip+aes
						# wpa-mixed aes
						# wpa-mixed tkip+aes
						case $tmp_enc in
							*2*aes|*-mixed*aes)
								if [ $radio -eq 0 ]; then
									tmp_config="$(uci get fastroaming.fastroaming_2G.enable)"
								else
									tmp_config="$(uci get fastroaming.fastroaming_5G.enable)"
								fi
							;;
							*)
								tmp_config=0
							;;
						esac
						echo "	option $uci_radio_token '$tmp_config'" >> $wifi_section_name
					;;
					"nss")
						if [ $radio -eq 1 ]; then
							echo "	option $uci_radio_token '$nss_value'" >> $wifi_section_name
						fi
					;;
					"macfilter")
						tmp_config="$(uci get wireless.w${radio}_index$((idx-1)).macpolicy)" 2>/dev/null
						[ ! -z "$tmp_config" ] && echo "	option $uci_radio_token '$tmp_config'" >> $wifi_section_name
					;;
					"allowmaclist"|"denymaclist")
						mac_list=""
						if [ $radio -eq 0 ]; then
							var="$(uci show filter | grep -w interface=$((idx-1)))" 2>/dev/null
						else
							var="$(uci show filter | grep -w interface=5$((idx-1)))" 2>/dev/null
						fi
						[ ! -z "$var" ] &&
						{
							for token in $var; do
								var="$(echo $token | cut -d"." -f2)"
								var1="filter."$var".mac"
								var2="filter."$var".type"
								mac_address="$(uci get $var1)"
								mac_policy="$(uci get $var2)"
								if [ "$mac_list" == "" ]; then
									mac_list=$mac_address
								else
									mac_list=$mac_list" "$mac_address
								fi
							done
							if [ "$mac_policy" == "1" -a "$uci_radio_token" == "allowmaclist" ]; then
								echo "	option allowmaclist '$mac_list'" >> $wifi_section_name
							elif [ "$mac_policy" == "2" -a "$uci_radio_token" == "denymaclist" ]; then	
								echo "	option denymaclist '$mac_list'" >> $wifi_section_name
							fi
						}
					;;
					"encryption")
						if [ "$temp_value" == "none" ]; then
							echo "	option encryption 'none'" >> $wifi_section_name
						else
							set_security_type_config "$temp_value" $radio $((idx-1))
						fi
						[ $is_support_radius_account -eq 1 ] && set_wpapsk_config $radio $((idx-1)) "wpa"
					;;
					"key_id")
						tmp_config="$(uci get wireless.w${radio}_index$((idx-1)).WepKeyIdx)" 2>/dev/null
						echo "	option $uci_radio_token '$tmp_config'" >> $wifi_section_name
					;;
					"nasid")
						echo "	option $uci_radio_token ''" >> $wifi_section_name
					;;
					"nasport")
						echo "	option $uci_radio_token '0'" >> $wifi_section_name
					;;
				esac
			}
		done

		set_normal_config

		# wifi0_ssid_1 / wifi1_ssid_1
		if [ $idx -eq 1 ]; then
			set_wps_config
		fi
		echo "" >> $wifi_section_name
	done
done

### WDS BR ###
[ "$model_name" == "enh1750ext" ] && {
	wds_br_convert
}

### WDS AP ###
[ "$model_name" == "enh1750ext" ] && {
	wds_ap_convert
}

### WDS STA ###
[ "$model_name" == "enh1750ext" ] && {
	wds_sta_convert
}

### CB mode ###
[ "$model_name" == "enh1750ext" ] && {
	# enh1750ext needs this part, otherwise GUI wireless page will be blank.
	cb_convert
}


###CB###
#wifi_wdssta_options="device network mode encryption bintval dtim_period frag preamble nawds nawds_encr wds disabled mode_display ifname ssid preferbssid_enable nss"
#for radio in `seq 0 1`; do
#	echo "config wifi-iface 'wifi${radio}_sta'" >> $wifi_section_name
#	for uci_radio_token in $wifi_wdssta_options; do
#		temp_value="$(uci show wireless.w${radio}_index19 | grep -w "${uci_radio_token}" | awk -F"=" '{ print $2 }')"
#		[ ! -z "$temp_value" -a "$uci_radio_token" != "ifname" ] && echo "	option $uci_radio_token '$temp_value'" >> $wifi_section_name || 
#		{
#			case $uci_radio_token in
#				"disabled")
#					tmp_config="$(uci get wireless.w${radio}_index19.WLANEnable)"
#					if [ $tmp_config -eq 0 ]; then
#						cnv_v='1'
#					else
#						cnv_v='0'
#					fi
#					echo "	option $uci_radio_token '$cnv_v'" >> $wifi_section_name
#					;;
#				"mode_display")
#					echo "	option $uci_radio_token 'sta'" >> $wifi_section_name
#					;;
#				"ifname")
#					if [ $radio -eq 0 ]; then
#						echo "	option $uci_radio_token 'ath26'" >> $wifi_section_name
#					else
#						echo "	option $uci_radio_token 'ath56'" >> $wifi_section_name
#					fi
#					;;
#				"preferbssid_enable")
#					tmp_config="$(uci get wireless.w${radio}_index19.PreferBSSIDEnable)"
#					echo "	option $uci_radio_token '$tmp_config'" >> $wifi_section_name
#					;;
#				"nawds")
#					echo "	option $uci_radio_token '0'" >> $wifi_section_name
#					;;
#			esac
#		}
#	done
#  set_normal_config "sta"
#	echo "" >> $wifi_section_name
#done

### GuestNetwork ###
for radio in `seq 0 1`; do
	echo "config wifi-iface 'wifi${radio}_guest'" >> $wifi_section_name
	
	for uci_radio_token in $wifi_guest_options; do
		temp_value="$(uci get wireless.w${radio}_index25."${uci_radio_token}" | sed s/\'/$sp_str/g)"
		[ ! -z "$temp_value" -a "$uci_radio_token" != "ifname" -a "$uci_radio_token" != "encryption" ] && 
		echo "	option $uci_radio_token '$temp_value'" >> $wifi_section_name || 
		{
			case $uci_radio_token in
				"bintval")
					echo "	option $uci_radio_token '400'" >> $wifi_section_name
				;;
				"dtim_period")
					echo "	option $uci_radio_token '2'" >> $wifi_section_name
				;;
				"frag")
					echo "	option $uci_radio_token '2346'" >> $wifi_section_name
				;;
				"preamble")
					echo "	option $uci_radio_token '0'" >> $wifi_section_name
				;;
				"nawds")
					echo "	option $uci_radio_token '0'" >> $wifi_section_name
				;;
				"disabled")
					tmp_config="$(uci get wireless.w${radio}_index25.WLANEnable)"
					if [ $tmp_config -eq 0 ]; then
						cnv_v='1'
					else
						cnv_v='0'
					fi
					echo "	option $uci_radio_token '$cnv_v'" >> $wifi_section_name
				;;
				"ifname")
					if [ $radio -eq 0 ]; then
						echo "	option $uci_radio_token 'ath27'" >> $wifi_section_name
					else
						echo "	option $uci_radio_token 'ath57'" >> $wifi_section_name
					fi
				;;
				"mode_display")
					echo "	option $uci_radio_token 'ap'" >> $wifi_section_name
				;;
				"mode_display_2")
					echo "	option $uci_radio_token 'wds_ap'" >> $wifi_section_name
				;;
				"l2_isolatior")
					tmp_config="$(uci get wireless.w${radio}_index25.L2_isolation)"
					echo "	option $uci_radio_token '$tmp_config'" >> $wifi_section_name
				;;
				"tc_enabled")
					echo "	option $uci_radio_token '0'" >> $wifi_section_name
				;;
				"mcastenhance")
					echo "	option $uci_radio_token '2'" >> $wifi_section_name
				;;
				"nss")
					if [ $radio -eq 1 ]; then
						echo "	option $uci_radio_token '$nss_value'" >> $wifi_section_name
					fi
				;;
				"encryption")
					if [ "$temp_value" == "none" ]; then
						echo "	option encryption 'none'" >> $wifi_section_name
					else
						set_security_type_config "$temp_value" $radio 25
					fi
					[ $is_support_radius_account -eq 1 ] && set_wpapsk_config $radio 25 "wpa"
				;;
				"status")
					echo "	option status '0'" >> $wifi_section_name
				;;
			esac
		}
	done
	echo "" >> $wifi_section_name
done



# Convert senao /etc/passwd
echo "Converting /etc/passwd ........" > /dev/console
senao_usr=$(cat /etc/webpasswd|cut -d':' -f1)
senao_passwd=$(cat /etc/passwd|grep -w "${senao_usr}"|cut -d':' -f2)
echo "root:x:0:0:root:/root:/bin/ash" > $rewrite_passwd
echo "daemon:*:1:1:daemon:/var:/bin/false" >> $rewrite_passwd
echo "ftp:*:55:55:ftp:/home/ftp:/bin/false" >> $rewrite_passwd
echo "network:*:101:101:network:/var:/bin/false" >> $rewrite_passwd
echo "nobody:*:65534:65534:nobody:/var:/bin/false" >> $rewrite_passwd
echo "$senao_usr:$senao_passwd:0:0:$senao_usr:/tmp:/bin/ash" >> $rewrite_passwd



# Convert senao /etc/shadow
#Note:
#		If there is no /etc/shadow in sysupgrade.tgz,
#		Smartopenwrt will use default /etc/shadow after fwupgrade, and its contains admin/admin,
#		so sshd(dropbear) admin's passwd will be set to default.
#Solution: Add a fake /etc/shadow without admin, and Smartopenwrt will use /etc/passwd.
echo "Converting /etc/shadow ........" > /dev/console
echo "root:x:0:0:99999:7:::" > $etc_shadow
echo "daemon:*:0:0:99999:7:::" >> $etc_shadow
echo "ftp:*:0:0:99999:7:::" >> $etc_shadow
echo "network:*:0:0:99999:7:::" >> $etc_shadow
echo "nobody:*:0:0:99999:7:::" >> $etc_shadow
